<?php 

namespace App\Helper;

/**
 * Logging class:
 * - contains lfile, lwrite and lclose public methods
 * - lfile sets path and name of log file
 * - lwrite writes message to the log file (and implicitly opens log file)
 * - lclose closes log file
 * - first call of lwrite method will open log file implicitly
 * - message is written with the following format: [d/M/Y:H:i:s] (script name) message
 */
class Logging {
    // declare log file and file pointer as private properties
    private $log_file, $fp;
    // set log file (path and name)
    public function lfile($path) {
    	
    	// in case of Windows set default log file
    	if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
    		$log_file_default = 'c:/php/logfile.txt';
    	}
    	// set default log file for Linux and other systems
    	else {
    		$log_file_default = '/tmp/logfile.txt';
    	}
    	// define log file from lfile method or use previously set default
    	$this->log_file  = $path? $path: $log_file_default;

    }
    // write message to the log file
    public function lwrite($message,$append=true) {
        // if file pointer doesn't exist, then open log file
        if (!is_resource($this->fp)) {
            $this->lopen($append);
        }
        // define script name
        $script_name = pathinfo($_SERVER['PHP_SELF'], PATHINFO_FILENAME);
        // define current time and suppress E_WARNING if using the system TZ settings
        // (don't forget to set the INI setting date.timezone)
        $time = @date('[d/M/Y:H:i:s]');
        // write current time, script name and message to the log file
        fwrite($this->fp, "$time ($script_name) $message" . PHP_EOL);
    }
    // close log file (it's always a good idea to close a file when you're done with it)
    public function lclose() {
        fclose($this->fp);
    }
    // open log file (private method)
    private function lopen($append=true) {
        
        // open log file for writing only and place file pointer at the end of the file
        // (if the file does not exist, try to create it)
        if($append){
        	$this->fp = fopen($this->log_file, 'a') or exit("Can't open $this->log_file!");
        }else{
        	$this->fp = fopen($this->log_file,'w') or exit("Can't open $this->log_file!");
        }
    }
    
    // file to read contents from the file
    public function lread(){
    	// if file pointer doesn't exist, then open log file
    	if (!is_resource($this->fp)) {
    		$this->lopen();
    	}
    	$contents = file_get_contents($this->log_file);
    	return $contents;
    }

}

?>
