<?php

namespace App\Http\Controllers;

header("Access-Control-Allow-Origin: *");
use App\Model\RechargeApi;
use App\Model\Shopify\Product;
use App\Model\GetPackCount;
use App\CheckoutFunctions\checkoutFunctions;
use App\Helper\helper;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class CheckoutController extends Controller
{

    private $modelObj;
    private $checkObj;
    private $packCount;
    private $logDirectory;
    private $logFiles;
    private $logPath;
    private $helperObj;
    private $checkoutReturn;
    private $productObj;
    private $totalPrice = 0;

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct() {

        $this->modelObj   = new RechargeApi();
        $this->packCount  = new GetPackCount();
        $this->productObj = new Product();
        $this->checkObj   = new checkoutFunctions();
        $this->helperObj  = new Helper();

        $this->setPaths();
        $this->getLogFilesList();
        $this->setWithLogFiles();
    }


    public function setPaths() {

        $this->logDirectory = dirname(__FILE__) . "/../../../storage/logs/";
    }

    public function getLogFilesList() {

        $this->logPath = $this->logDirectory . date('dmY');

        if (!is_dir($this->logPath)) {
            //Directory does not exist, so lets create it.
            mkdir($this->logPath);
            chmod($this->logPath, 0777);

        }

        $this->logFiles = array(
            'log'             => array(
                "type"     => "text",
                "filename" => $this->logPath . "/log.txt",
                "mode"     => ""),
                'checkoutDetails' => array(
                "type"     => "csv",
                "filename" => $this->logDirectory . "checkoutOrderDetails.csv",
                "mode"     => "a",
                "columns"  => $this->checkObj->setCheckoutColumns()),
        );
    }

    public function setWithLogFiles() {

        if (!empty($this->logFiles)) {
            foreach ($this->logFiles as $fileDetails) {
                $fileName = $fileDetails['filename'];
                if ($fileDetails['type'] == 'csv' && !file_exists($fileName)) {
                    $handler = fopen($fileName, $fileDetails['mode']);
                    fputcsv($handler, $fileDetails['columns']);
                    fclose($handler);
                    touch($fileName);
                    //Make it writeable
                    chmod($fileName, 0777);
                }
            }

        }
    }

    /**
     * Create checkout
     */

    public function CreateCheckout() {

        if (empty($_REQUEST['data'])) {

            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo json_encode($response);
            die();

        }

        $requestData             = $_REQUEST['data'];
        $parameterLineItemStatus = 0;
        $parameterShippingStatus = 0;
        $checkoutRequest         = array();
        $lineItems               = array();
        $shippingAddress         = array();

        $receivedData = json_decode($requestData);

        $receivedData = $this->helperObj->objectToArray($receivedData);
        
        $otp = false;
        $otpExtra = 0.9;

        if(isset($receivedData['checkout']['one_time_purchase'])){
            $otp = trim($receivedData['checkout']['one_time_purchase']);
        }
        // if(isset($receivedData['checkout']['one_time_purchase'])){
        //     $otpExtra = trim($receivedData['checkout']['otp_extra']);
        // }

        if (isset($receivedData['checkout']['line_items'])) {

            foreach ($receivedData['checkout']['line_items'] as $checkoutValue) {

                foreach ($checkoutValue['properties'] as $propertyValue) {

                    if ((empty($propertyValue['variant_id']) || empty($propertyValue['product_id'])
                        || empty($propertyValue['name']) || empty($propertyValue['value']))
                        && !isset($propertyValue['utm'])
                        && !isset($propertyValue['override_free_shipping'])
                        && !isset($propertyValue['special_pack_type'])
                        && !isset($propertyValue['partner'])
                        && !isset($propertyValue['campaign'])
                        && !isset($propertyValue['first_name'])
                        && !isset($propertyValue['source'])
                        && !isset($propertyValue['medium'])
                        && !isset($propertyValue['content'])
                        && !isset($propertyValue['term'])
                        && !isset($propertyValue['quiz_id'])
                        && !isset($propertyValue['campaign_id'])
                        && !isset($propertyValue['template_id'])
                        && !isset($propertyValue['future_month_free'])
                        ) {

                        $parameterLineItemStatus = 1;

                    }
                }
            }
        } else {
            $parameterLineItemStatus = 1;
        }


        if (empty($receivedData['checkout']['line_items']) || $parameterLineItemStatus == 1) {

            $response = array(
                "status"       => "Failed",
                "errorMessage" => "Missing required parameters in the line Item properties",
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }

        if (!empty($receivedData['checkout']['shipping_address'])) {

            if (empty($receivedData['checkout']['shipping_address']['first_name'])
                || empty($receivedData['checkout']['shipping_address']['last_name'])
                || empty($receivedData['checkout']['shipping_address']['address1'])
                || empty($receivedData['checkout']['shipping_address']['city'])
                || empty($receivedData['checkout']['shipping_address']['country'])
                || empty($receivedData['checkout']['shipping_address']['province'])) {

                $parameterShippingStatus = 1;

            }
        }

        if ($parameterShippingStatus == 1) {

            $response = array(
                "status"       => "Failed",
                "errorMessage" => "Missing required parameters in the Shipping Address",
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }

        $utmParameter = '';
        $shippingOverRideStatus = '';
        $specialPackType = '';
        $partnerValue = '';
        $campaignValue = '';
        $firstName = '';
        $source = '';
        $medium = '';
        $content = '';
        $term = '';
        $quizId = '';
        $campaignId = '';
        $templateId = '';
        $futureMonthFree = '';
        

        if (isset($receivedData['checkout']['line_items'])) {
            $i = 0;
            foreach ($receivedData['checkout']['line_items'] as $itemValue) {
                if (isset($itemValue['properties'])) {
                    //Handling duplicate line item in properties
                    $rearrangeProperties = array();
                    $j = 1;
                    foreach ($itemValue['properties'] as $propertyValue) {
                        if(!isset($propertyValue['utm'])
                        && !isset($propertyValue['override_free_shipping'])
                        && !isset($propertyValue['special_pack_type'])
                        && !isset($propertyValue['partner'])
                        && !isset($propertyValue['campaign'])
                        && !isset($propertyValue['first_name'])
                        && !isset($propertyValue['source'])
                        && !isset($propertyValue['medium'])
                        && !isset($propertyValue['content'])
                        && !isset($propertyValue['term'])
                        && !isset($propertyValue['quiz_id'])
                        && !isset($propertyValue['campaign_id'])
                        && !isset($propertyValue['template_id'])
                        && !isset($propertyValue['future_month_free'])
                        ) {
                            if(isset($rearrangeProperties[$propertyValue['variant_id']])){
                                $rearrangeProperties[$propertyValue['variant_id']]['quantity'] +=
                                $propertyValue['quantity'];

                            } else {
                                $propertyValue['name'] = 'Vitamin'. ($j<10? '0'.$j++:$j++);
                                $rearrangeProperties[$propertyValue['variant_id']] = $propertyValue;
                         }
                       } else {
                            if (isset($propertyValue['utm'])) {

                                $utmParameter = $propertyValue['utm'];
                            } elseif (isset($propertyValue['override_free_shipping'])) {

                                $shippingOverRideStatus = ($propertyValue['override_free_shipping'] == "true") ?
                                "true" : "false";
                            } elseif (isset($propertyValue['special_pack_type'])) {

                                $specialPackType = $propertyValue['special_pack_type'];

                            } elseif (isset($propertyValue['partner'])) {

                                $partnerValue = $propertyValue['partner'];

                            } elseif (isset($propertyValue['campaign'])) {

                                $campaignValue = $propertyValue['campaign'];

                            } elseif (isset($propertyValue['first_name'])) {

                                $firstName = $propertyValue['first_name'];

                            } elseif (isset($propertyValue['source'])) {

                                $source = $propertyValue['source'];

                            } elseif (isset($propertyValue['medium'])) {

                                $medium = $propertyValue['medium'];

                            } elseif (isset($propertyValue['content'])) {

                                $content = $propertyValue['content'];

                            } elseif (isset($propertyValue['term'])) {

                                $term = $propertyValue['term'];

                            } elseif (isset($propertyValue['quiz_id'])) {

                                $quizId = $propertyValue['quiz_id'];
                                
                            } elseif (isset($propertyValue['campaign_id'])) {

                                $campaignId = $propertyValue['campaign_id'];

                            } elseif (isset($propertyValue['template_id'])) {

                                $templateId = $propertyValue['template_id'];

                            }elseif (isset($propertyValue['future_month_free'])) {

                                $futureMonthFree = $propertyValue['future_month_free'];

                            }
                        }
                    }
                    $itemValue['properties'] =  array_values($rearrangeProperties);
                }
                unset($receivedData['checkout']['line_items'][$i]);
                $receivedData['checkout']['line_items'][$i] =  $itemValue;
                $i++;
            }
        }

        $lineItemProperties = array();
        $packData = '';
        $packCountRequest = array();
        $productPackCount = '';
        $melatoninProductIds = [1837729808455, 1837729775687, 4329622405191];
        $melatoninCount = 0;
        $productVariantErrorStatus = 0;
        $variantError = '';
        $productValue = '';

        if (isset($receivedData['checkout']['line_items'])) {
            
            if( ($otp || $otp == 'true') && $otpExtra > 0){
              $otpEmail = !empty($receivedData['checkout']['email']) ? $receivedData['checkout']['email'] : '';
              $otpExtraInFile = $this->getOTPExtra($otpEmail);
              if($otpExtraInFile != ''){
                  $otpExtraInFile = (100-$otpExtraInFile)/100;
              }
              
              if(($otpExtraInFile != '' && $otpExtraInFile > 0) && ($otpExtraInFile != $otpExtra)){
                  $otpExtra = $otpExtraInFile;
              }
            }

            foreach ($receivedData['checkout']['line_items'] as $itemValue) {

                if (isset($itemValue['properties'])) {

                    foreach ($itemValue['properties'] as $propertyValue) {

                        if (!empty($propertyValue['product_id']) && !empty($propertyValue['variant_id'])) {

                            $variantDetails = $this->productObj->getShopifyProductByID(
                                $propertyValue['product_id'], $propertyValue['variant_id']);

                            if (!empty($variantDetails['errors']) || !empty($variantDetails['error'])) {

                                $productVariantErrorStatus = 1;
                                $productValue = $propertyValue['value'];
                                $variantError = !empty($variantDetails['errors']) ? $variantDetails['errors'] :
                                $variantDetails['error'];
                                break;
                            }
                            $freePill = false;
                            // if ($variantDetails['variant']['sku'] == "A062AB") {
                            //     $freePill = true;
                            // }
                            $lineItemProperties[$propertyValue['name']] = $propertyValue['value'] .
                                (!empty($propertyValue['dosage']) ? ' | ' . $propertyValue['dosage'] : '') .
                                (!empty($propertyValue['quantity']) ? ' | ' . $propertyValue['quantity'] : ' | 1') .
                                ' | ' . $propertyValue['variant_id'] . ' | ' . $variantDetails['variant']['sku'] . ($freePill ? ' | free_pill' : '');

                            if (empty($propertyValue['quantity'])) {

                                $productQuantity = 1;

                            } else {

                                $productQuantity = $propertyValue['quantity'];
                            }

                            if(in_array($propertyValue['product_id'], $melatoninProductIds)) {

                                $melatoninCount = $melatoninCount + $productQuantity;

                            }

                            array_push($packCountRequest, array("bulkID" =>
                            $variantDetails['variant']['sku'], "qty" => $productQuantity));
                            if ($freePill) {
                                continue;
                            }
                            if (isset($variantDetails['variant']['price'])) {
                                
                                if( ($otp || $otp == 'true') && $otpExtra > 0){
                                  
                                  $this->totalPrice = $this->totalPrice + (($variantDetails['variant']['price'] * $productQuantity)/$otpExtra);
                                
                                }else{
                                   $this->totalPrice = $this->totalPrice +  ($variantDetails['variant']['price'] * $productQuantity);
                                }

                            }
                        }
                    }
                }
            }
        }

        if ($productVariantErrorStatus == 1) {

            $response = array(
                "status"       => "Failed",
                "errorMessage" => (strtolower($variantError) == 'not found') ?
                "ProductID or variantID is invalid of product - ".$productValue : $variantError,
            );
            $errorMessage = "Error - ".$response['errorMessage'].', Shopify API name:p80w-checkout';

            $this->helperObj->logMe($errorMessage, $this->logFiles['log']['filename']);
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }

        if(isset($packCountRequest)) {
            $packCount = $this->packCount->getProductPackCount($packCountRequest);
        }

        if(isset($packCount)) {
            $productPackCount = !empty($packCount->body->Pack) ?
            "{$packCount->body->Pack}" : "{$itemValue['Packs']}";
        }

        if (!empty($productPackCount)) {
            $lineItemProperties['Packs'] = $productPackCount;
        }

        if ($melatoninCount > 4) {
            $status = true;
            $lineItemProperties['melatonin_warning'] = "{$status}";
        }

        if (!empty($utmParameter)) {
            $lineItemProperties['utm'] = $utmParameter;
        }

        if (!empty($shippingOverRideStatus)) {
            $lineItemProperties['override_free_shipping'] = $shippingOverRideStatus;
        }

        if (!empty($specialPackType)) {
            $lineItemProperties['special_pack_type'] = $specialPackType;
        }

        if (!empty($partnerValue)) {
            $lineItemProperties['partner'] = $partnerValue;
        }

        if (!empty($campaignValue)) {
            $lineItemProperties['campaign'] = $campaignValue;
        }

        if (!empty($firstName)) {
            $lineItemProperties['first_name'] = $firstName;
        }

        if (!empty($source)) {
            $lineItemProperties['source'] = $source;
        }

        if (!empty($medium)) {
            $lineItemProperties['medium'] = $medium;
        }

        if (!empty($content)) {
            $lineItemProperties['content'] = $content;
        }

        if (!empty($term)) {
            $lineItemProperties['term'] = $term;
        }

        if (!empty($quizId)) {
            $lineItemProperties['quiz_id'] = $quizId;
        }
        
        if (!empty($campaignId)) {
            $lineItemProperties['campaign_id'] = $campaignId;
        }

        if (!empty($templateId)) {
            $lineItemProperties['template_id'] = $templateId;
        }
        
        if (!empty($futureMonthFree)) {
            $lineItemProperties['future_month_free'] = $futureMonthFree;
        }
        
        if($otp || $otp == 'true'){
            $lineItemProperties['one_time_purchase'] = 'true';
        }

        if (isset($receivedData['checkout']['line_items'])) {

            foreach ($receivedData['checkout']['line_items'] as $checkoutValue) {

                $lineItemTemp = array(
                    "price"                                   => !empty($this->totalPrice) ? $this->totalPrice : '',
                    "product_id"                              => !empty($checkoutValue['product_id']) ? $checkoutValue['product_id'] : '',
                    "properties"                              => isset($lineItemProperties) ? $lineItemProperties : '',
                    "quantity"                                => !empty($checkoutValue['quantity']) ? $checkoutValue['quantity'] : 1,
                    "variant_id"                              => $checkoutValue['variant_id'],
                    // "charge_interval_frequency"               => $checkoutValue['charge_interval_frequency'],
                    // "order_interval_frequency"                => $checkoutValue['order_interval_frequency'],
                    // "order_interval_unit"                     => $checkoutValue['order_interval_unit'],
                    "cutoff_day_of_month"                     => !empty($checkoutValue['cutoff_day_of_month'])
                        ? $checkoutValue['cutoff_day_of_month'] : '',
                    "cutoff_day_of_week"                      => !empty($checkoutValue['cutoff_day_of_week'])
                        ? $checkoutValue['cutoff_day_of_week'] : '',
                    "expire_after_specific_number_of_charges" => !empty($checkoutValue['expire_after_specific_number_of_charges'])
                        ? $checkoutValue['expire_after_specific_number_of_charges'] : '',
                    "fulfillment_service"                     => "manual",
                    "grams"                                   => !empty($checkoutValue['grams']) ? $checkoutValue['grams'] : '',
                    "line_price"                              => !empty($checkoutValue['line_price']) ? $checkoutValue['line_price'] : '',
                    "order_day_of_month"                      => !empty($checkoutValue['order_day_of_month'])
                        ? $checkoutValue['order_day_of_month'] : '',
                    "order_day_of_week"                       => !empty($checkoutValue['order_day_of_week'])
                        ? $checkoutValue['order_day_of_week'] : '',
                    //"requires_shipping" => true,
                    "sku"                                     => !empty($checkoutValue['sku']) ? $checkoutValue['sku'] : '',
                    //"taxable" => true,
                    "title"                                   => !empty($checkoutValue['title']) ? $checkoutValue['title'] : '',
                    "variant_title"                           => !empty($checkoutValue['variant_title'])
                        ? $checkoutValue['variant_title'] : '',
                    "vendor"                                  => !empty($checkoutValue['vendor']) ? $checkoutValue['vendor'] : '',
                );
                
                if(!$otp || $otp == 'false'){
                    $lineItemTemp["charge_interval_frequency"] = $checkoutValue['charge_interval_frequency'];
                    $lineItemTemp["order_interval_frequency"]  = $checkoutValue['order_interval_frequency'];
                    $lineItemTemp["order_interval_unit"]       = $checkoutValue['order_interval_unit'];
                }

                foreach ($lineItemTemp as $key => $value) {
                    if (empty($value)) {
                        unset($lineItemTemp[$key]);
                    }
                }
                
                // print_r($lineItemTemp);die;

                array_push($lineItems, $lineItemTemp);
            }

        }

        if (!empty($receivedData['checkout']['shipping_address'])) {

            $shippingAddress = array(
                "address1"   => !empty($receivedData['checkout']['shipping_address']['address1'])
                    ? $receivedData['checkout']['shipping_address']['address1'] : '',
                "address2"   => !empty($receivedData['checkout']['shipping_address']['address2'])
                    ? $receivedData['checkout']['shipping_address']['address2'] : '',
                "city"       => !empty($receivedData['checkout']['shipping_address']['city'])
                    ? $receivedData['checkout']['shipping_address']['city'] : '',
                "company"    => !empty($receivedData['checkout']['shipping_address']['company'])
                    ? $receivedData['checkout']['shipping_address']['company'] : '',
                "country"    => !empty($receivedData['checkout']['shipping_address']['country'])
                    ? $receivedData['checkout']['shipping_address']['country'] : '',
                "first_name" => !empty($receivedData['checkout']['shipping_address']['first_name'])
                    ? $receivedData['checkout']['shipping_address']['first_name'] : '',
                "last_name"  => !empty($receivedData['checkout']['shipping_address']['last_name'])
                    ? $receivedData['checkout']['shipping_address']['last_name'] : '',
                "phone"      => !empty($receivedData['checkout']['shipping_address']['phone'])
                    ? $receivedData['checkout']['shipping_address']['phone'] : '',
                "province"   => !empty($receivedData['checkout']['shipping_address']['province'])
                    ? $receivedData['checkout']['shipping_address']['province'] : '',
                "zip"        => !empty($receivedData['checkout']['shipping_address']['zip'])
                    ? $receivedData['checkout']['shipping_address']['zip'] : ''

            );

            foreach ($shippingAddress as $key => $value) {
                if (empty($value)) {
                    unset($shippingAddress[$key]);
                }
            }
        }

        $checkoutRequest = array(
            "checkout" => array(
                "line_items"               => $lineItems,
                "shipping_address"         => $shippingAddress,
                "email"                    => !empty($receivedData['checkout']['email']) ? $receivedData['checkout']['email'] : '',
                "external_checkout_id"     => !empty($receivedData['checkout']['external_checkout_id'])
                    ? $receivedData['checkout']['external_checkout_id'] : '',
                "external_checkout_source" => !empty($receivedData['checkout']['external_checkout_source'])
                    ? $receivedData['checkout']['external_checkout_source'] : '',
                "discount_code"            => !empty($receivedData['checkout']['discount_code'])
                    ? $receivedData['checkout']['discount_code'] : '',

            )
        );

        foreach ($checkoutRequest['checkout'] as $key => $value) {
            if (empty($value)) {
                unset($checkoutRequest['checkout'][$key]);
            }
        }
        
        $this->helperObj->logMe("Request => ".json_encode($checkoutRequest),$this->logFiles['log']['filename']);

        $checkoutReturn = $this->modelObj->createCheckout(json_encode($checkoutRequest));
        
        $this->helperObj->logMe("Response => ".json_encode($checkoutReturn),$this->logFiles['log']['filename']);
        

        if (isset($checkoutReturn->error) || isset($checkoutReturn->errors) ||
            empty($checkoutReturn)) {

            $status = "failed";
            $error  = "Unknown error";
            if (isset($checkoutReturn->error)) {

                $error = $checkoutReturn->error;

            } else if (isset($checkoutReturn->errors)) {

                $errorToArray = $this->helperObj->objectToArray($checkoutReturn->errors);
                $error        = "The required parameter variant_id or quantity is missing or the request is not correct";

            }

            $this->helperObj->logMe($error, $this->logFiles['log']['filename']);
            $response = array(
                "status"       => "failed",
                "message"      => "Failed to create the Checkout",
                "errorMessage" => $error,
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die;
        }

        $url = env('NURISH_CHECKOUT_URL', true) . $checkoutReturn->checkout->token .
            '?myshopify_domain=' . env('SHOPIFY_STORE', true);
        echo $_REQUEST['callback'] . '({"status":"success","url": "' . $url . '",
            "override_free_shipping": "' .$shippingOverRideStatus. '",
            "special_pack_type": "'.$specialPackType.'","partner": "'.$partnerValue.'",
            "campaign": "'.$campaignValue.'","first_name": "'.$firstName.'",
            "source": "'.$source.'","medium": "'.$medium.'","content": "'.$content.'",
            "term": "'.$term.'","quiz_id": "'.$quizId.'","campaign_id": "'.$campaignId.'",
            "template_id": "'.$templateId.'","future_month_free": "'.$futureMonthFree.'", "errorMessage":""})';
        die();

    }

    /**
     * Update checkout
     */
    public function updateSubscriptionLineItemProperties() {

        if (empty($_REQUEST['data'])) {

            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"   => $status,
                "message" => $message,
                "error" => $error
            );
            echo json_encode($response);
            die();
        }
        $requestData = $_REQUEST['data'];

        $receivedData = json_decode($requestData);

        if (empty($receivedData->subscription_id)) {


            $response = array(
                "status"       => "Failed",
                "errorMessage" => "The subscription id is missing in the request",
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }

        $propertyStatus = 0;

        if (empty($receivedData->properties)) {

            $propertyStatus = 1;

        } else {

            foreach ($receivedData->properties as $Value) {

                if ((empty($Value->product_id) || empty($Value->variant_id) ||
                empty($Value->name) || empty($Value->value))
                && empty($Value->utm)
                && !isset($Value->override_free_shipping)
                && empty($Value->special_pack_type)
                && empty($Value->partner)
                && empty($Value->campaign)
                && empty($Value->first_name)
                && empty($Value->source)
                && empty($Value->medium)
                && empty($Value->content)
                && empty($Value->term)
                && empty($Value->quiz_id)) {

                    $propertyStatus = 1;
                }
            }
        }

        if ($propertyStatus == 1) {

            $response = array(
                "status"       => "Failed",
                "errorMessage" => "Missing required parameters in the line item properties",
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }

        if (isset($receivedData->properties)) {
            $rearrangeProperties = array();
            $i = 0;
            $j = 1;

            foreach ($receivedData->properties as $propertyValue) {
                $i++;
                if(!empty($propertyValue->variant_id)) {
                if(isset($rearrangeProperties[$propertyValue->variant_id])) {
                   $rearrangeProperties[$propertyValue->variant_id]->quantity += $propertyValue->quantity;


                }else{
                    $propertyValue->name =  'Vitamin'. ($j<10? '0'.$j++:$j++);
                    $rearrangeProperties[$propertyValue->variant_id] = $propertyValue;
                }
              } else{
                  //$rearrangeProperties['pack'.$i] =$propertyValue;

                if (!empty($propertyValue->utm)) {

                    $rearrangeProperties['utm'] =  (object)["name" => "utm","value" => $propertyValue->utm];
                } elseif (isset($propertyValue->override_free_shipping)) {

                    $rearrangeProperties['override_free_shipping'] = ($propertyValue->override_free_shipping == "true") ?
                    (object)["name" => "override_free_shipping", "value" => "true"] :
                    (object)["name" => "override_free_shipping", "value" => "false"];
                } elseif (!empty($propertyValue->special_pack_type)) {

                    $rearrangeProperties['special_pack_type'] =
                    (object)["name" => "special_pack_type","value" => $propertyValue->special_pack_type];

                } elseif (!empty($propertyValue->partner)) {

                    $rearrangeProperties['partner'] =
                    (object)["name" => "partner","value" => $propertyValue->partner];

                } elseif (!empty($propertyValue->campaign)) {

                    $rearrangeProperties['campaign'] =
                    (object)["name" => "campaign","value" => $propertyValue->campaign];

                } elseif (!empty($propertyValue->first_name)) {

                    $rearrangeProperties['first_name'] =
                    (object)["name" => "first_name","value" => $propertyValue->first_name];

                } elseif (!empty($propertyValue->source)) {

                    $rearrangeProperties['source'] =
                    (object)["name" => "source","value" => $propertyValue->source];

                } elseif (!empty($propertyValue->medium)) {

                    $rearrangeProperties['medium'] =
                    (object)["name" => "medium","value" => $propertyValue->medium];

                } elseif (!empty($propertyValue->content)) {

                    $rearrangeProperties['content'] =
                    (object)["name" => "content","value" => $propertyValue->content];

                } elseif (!empty($propertyValue->term)) {

                    $rearrangeProperties['term'] =
                    (object)["name" => "term","value" => $propertyValue->term];

                } elseif (!empty($propertyValue->quiz_id)) {

                    $rearrangeProperties['quiz_id'] =
                    (object)["name" => "quiz_id","value" => $propertyValue->quiz_id];
                }
              }
            }

            unset($receivedData->properties);
            $receivedData->properties =    array_values($rearrangeProperties);
        }

        $lineItemProperty = array();
        $productSKU       = array();
        $packCountRequest = array();
        $melatoninProductIds = [1837729808455, 1837729775687, 4329622405191];
        $melatoninCount = 0;
        $productVariantErrorStatus = 0;
        $variantError = '';
        $productValue = '';

        if (!empty($receivedData->properties)) {

            foreach ($receivedData->properties as $itemValue) {

                if (!empty($itemValue->product_id) && !empty($itemValue->variant_id)) {

                    $variantDetails = $this->productObj->getShopifyProductByID(
                        $itemValue->product_id, $itemValue->variant_id);

                    if (!empty($variantDetails['errors']) || !empty($variantDetails['error'])) {

                        $productVariantErrorStatus = 1;
                        $productValue = $itemValue->value;
                        $variantError = !empty($variantDetails['errors']) ? $variantDetails['errors'] :
                        $variantDetails['error'];
                        break;
                    }

                    $productSKU[$itemValue->variant_id] = $variantDetails['variant']['sku'];

                    if (empty($itemValue->quantity)) {

                        $productQuantity = 1;

                    } else {

                        $productQuantity = $itemValue->quantity;
                    }

                    if(in_array($itemValue->product_id, $melatoninProductIds)) {

                        $melatoninCount = $melatoninCount + $productQuantity;

                    }

                    array_push($packCountRequest, array("bulkID" =>
                    $variantDetails['variant']['sku'], "qty" => $productQuantity));
                    if (isset($itemValue->free_pill) && !empty($itemValue->free_pill)) {
                        continue;
                    }

                    if (isset($variantDetails['variant']['price'])) {

                        $this->totalPrice = $this->totalPrice + ($variantDetails['variant']['price'] * $productQuantity);
                    }
                }
            }
        }

        if ($productVariantErrorStatus == 1) {

            $response = array(
                "status"       => "Failed",
                "errorMessage" => (strtolower($variantError) == 'not found') ?
                "ProductID or variantID is invalid of product - ".$productValue : $variantError,
            );
            $errorMessage = "Error - ".$response['errorMessage'].', Shopify API name:p80w-checkout';

            $this->helperObj->logMe($errorMessage, $this->logFiles['log']['filename']);
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }

        $productPackCount = '';

        if(isset($packCountRequest)) {
            $packCount = $this->packCount->getProductPackCount($packCountRequest);
        }

        if(isset($packCount)) {
            $productPackCount = !empty($packCount->body->Pack) ? "{$packCount->body->Pack}" : '';
        }

        foreach ($receivedData->properties as $propertyValue) {

            $value = '';
            $quantity = 1;

            if (isset($propertyValue->product_id) && isset($propertyValue->variant_id)) {
                $freePill = '';
                if (isset($propertyValue->free_pill) && !empty($propertyValue->free_pill)) {
                    $freePill = ' | free_pill';
                }

                if (!empty($propertyValue->quantity)) {

                    $quantity = $propertyValue->quantity;

                    $value = $propertyValue->value . (!empty($propertyValue->dosage) ?
                            ' | ' . $propertyValue->dosage : ' | '. 'Default Title') . ' | ' . $quantity . ' | ' .
                        $propertyValue->variant_id . ' | ' . $productSKU[$propertyValue->variant_id] . $freePill;

                } else {

                    $value = $propertyValue->value . (!empty($propertyValue->dosage) ?
                            ' | ' . $propertyValue->dosage : ' | '. 'Default Title') . ' | ' . $quantity . ' | ' .
                        $propertyValue->variant_id . ' | ' . $productSKU[$propertyValue->variant_id] . $freePill;
                }

                $property = array(
                    "name"  => $propertyValue->name,
                    "value" => $value
                );

                array_push($lineItemProperty, $property);

            } else {

                $property = array(
                    "name"  => $propertyValue->name,
                    "value" => $propertyValue->value
                );

                array_push($lineItemProperty, $property);

            }
        }

        if(!empty($productPackCount)){

            $packCountArr = array(
                "name"  => "Packs",
                "value" => $productPackCount
            );

            array_push($lineItemProperty, $packCountArr);

        }

        if($melatoninCount > 4) {

            $melatoninWarning = array(
                "name"  => "melatonin_warning",
                "value" => true
            );
            array_push($lineItemProperty, $melatoninWarning);

        }

        $subscriptionID = $receivedData->subscription_id;
        if (!empty($subscriptionID)) {
            $subscriptionDetails = $this->modelObj->getSubscription($subscriptionID);
        }

        $lineItemProperties = [];
        $addedProperties = [];
        $vitaminUpdateStatus = 0;
        if (!empty($subscriptionDetails->subscription->properties)) {

            $subscriptionDetailsInArr = $this->helperObj->objectToArray($subscriptionDetails);
            foreach ($subscriptionDetailsInArr['subscription']['properties'] as $subKey => $subPropertyValue) {

                $changedPropertyData = '';
                $vitaminProductDetails = [];
                foreach ($lineItemProperty as $lineKey => $lineItemValue) {

                    $vitaminNameProperty = strtolower(trim($lineItemValue['name']));
                    if (strtolower(trim($subPropertyValue['name'])) == strtolower(trim($lineItemValue['name']))
                    && strpos($vitaminNameProperty, 'vitamin') === false) {

                        $changedPropertyData = $lineItemValue;
                    }
                    if (strpos($vitaminNameProperty, 'vitamin') !== false) {

                        $vitaminProductDetails[] = $lineItemValue;
                    }
                }

                $vitaminName = strtolower(trim($subPropertyValue['name']));
                if (!empty($changedPropertyData)) {
                    $lineItemProperties[] = $changedPropertyData;
                    $addedProperties[] = strtolower($changedPropertyData['name']);
                } else {
                    if(strpos($vitaminName, 'vitamin') === false) {
                        $lineItemProperties[] = $subPropertyValue;
                        $addedProperties[] = strtolower($subPropertyValue['name']);
                    }
                }
                if(strpos($vitaminName, 'vitamin') !== false && $vitaminUpdateStatus == 0) {
                    foreach ($vitaminProductDetails as $vitaminValue) {
                        $lineItemProperties[] = $vitaminValue;
                    }
                    $vitaminUpdateStatus = 1;
                }
            }
        }

        if (isset($addedProperties) && !empty($addedProperties)) {
            $notAddedProperty = [];
            foreach ($lineItemProperty as $lineKey => $lineItemValue) {

                $vitaminNameProperty = strtolower(trim($lineItemValue['name']));
                if (in_array($vitaminNameProperty, $addedProperties) == false
                && strpos($vitaminNameProperty, 'vitamin') === false) {

                    $notAddedProperty[] = $lineItemValue;
                }
            }

            if (!empty($notAddedProperty)) {

                foreach ($notAddedProperty as $notAddedValue) {

                    $lineItemProperties[] = $notAddedValue;
                }
            }
        }

        $updateRequest = array(
            "price"      => $this->totalPrice,
            "properties" => $lineItemProperties
        );

        $usShippingRateAdjustableCountries = array(

            "alabama","arizona","arkansas","california","colorado","connecticut","delaware","florida","georgia",
            "idaho","illinois","indiana","iowa","kansas","kentucky","louisiana","maine","maryland","massachusetts",
            "michigan","minnesota","mississippi","missouri","montana","nebraska","nevada","new-hampshire",
            "new-jersey","new-mexico","new-york","north-carolina","north-dakota","ohio","oklahoma",
            "oregon","pennsylvania","rhode-island","south-carolina","south-dakota","tennessee","texas",
            "utah","vermont","virginia","washington","west-virginia","wisconsin","wyoming"

        );

        $usShippingRateNonAdjustableCountries = array(
            "alaska","hawaii"
        );

        $shopifyDomesticShippingPrice = '';
        $shopifyNonDomesticShippingPrice = '';

        $shippingZones = $this->productObj->getShopifyShippingZone();

        if (isset($shippingZones['shipping_zones'])) {

            foreach ($shippingZones['shipping_zones'] as $shipValue) {

                $shippingZoneName = strtolower(trim($shipValue['name']));
                if (strpos($shipValue['name'], ",") !== false ) {

                    $shippingZoneName = str_replace(" ", "", $shippingZoneName);
                    $shippingZoneName = str_replace(",", "-", $shippingZoneName);
                }

                if ($shippingZoneName == 'domestic' || $shippingZoneName == 'alaska-hawaii') {

                    if(!empty($shipValue['price_based_shipping_rates'])) {

                        foreach ($shipValue['price_based_shipping_rates'] as $priceValue) {

                            $shippingName = strtolower(trim($priceValue['name']));
                            $shippingName = str_replace(" ", "-", $shippingName);

                            if($shippingName == 'standard-shipping') {

                                if ($shippingZoneName == 'domestic') {

                                    $shopifyDomesticShippingPrice = $priceValue['price'];

                                } else {

                                    $shopifyNonDomesticShippingPrice = $priceValue['price'];

                                }
                            }
                        }
                    }
                }
            }
        }

        if (!empty($subscriptionDetails->subscription->address_id)) {

            $userAddressDetails = $this->modelObj->getAddressByAddressID($subscriptionDetails->subscription->address_id);

        }

        $existingShippingPrice = '';

        if (!empty($userAddressDetails->address->original_shipping_lines)) {

        foreach ($userAddressDetails->address->original_shipping_lines as $shipPriceValue) {

            $shippingName = strtolower(trim($shipPriceValue->title));
            $shippingName = str_replace(" ", "-", $shippingName);

            if($shippingName == 'standard-shipping') {

                $existingShippingPrice = $shipPriceValue->price;

            }
          }
        }

        if (!empty($userAddressDetails->address->province)){

            $addressProvince = strtolower(trim($userAddressDetails->address->province));
            $addressProvince = str_replace(" ", "-", $addressProvince);

        } else {

            $addressProvince = '';
        }

        $shippingUpdateStatus = 0;
        $shippingPrice = '';
        $shippingCode = '';

        if (!empty($addressProvince) && in_array($addressProvince, $usShippingRateAdjustableCountries)) {

            if ($this->totalPrice >= 20) {
                $shippingPrice = 0;
                $shippingCode = "Free Shipping";
            } else {
                $shippingPrice = !empty($shopifyDomesticShippingPrice) ? $shopifyDomesticShippingPrice : 8;
                $shippingCode = "Standard Shipping";
            }

            $shippingUpdateStatus = 1;

        } elseif (!empty($addressProvince) && in_array($addressProvince, $usShippingRateNonAdjustableCountries)) {

            $shippingPrice = !empty($shopifyNonDomesticShippingPrice) ? $shopifyNonDomesticShippingPrice : 8;
            $shippingCode = "Standard Shipping";

            $shippingUpdateStatus = 1;

        } else {

            $shippingPrice = !empty($shopifyDomesticShippingPrice) ? $shopifyDomesticShippingPrice : 8;
            $shippingCode = "Standard Shipping";

            $shippingUpdateStatus = 1;

        }

        if ($shippingUpdateStatus == 1 && isset($subscriptionDetails) &&
        !empty($subscriptionDetails->subscription->address_id)) {

            $shippingUpdateRequest = array(

                "shipping_lines_override" => array(
                   array(
                    "price" => $shippingPrice,
                    "code"  => $shippingCode,
                    "title" => "Standard Shipping"

                )
              )
            );

            $updatedShippingRateResponse = $this->modelObj->updateShippingRateByAddress(
                $subscriptionDetails->subscription->address_id, json_encode($shippingUpdateRequest));

                if (isset($updatedShippingRateResponse->error) || isset($updatedShippingRateResponse->errors) ||
                empty($updatedShippingRateResponse)) {

                $status = "failed";
                $error  = "Unknown error";
                if (isset($updatedShippingRateResponse->error)) {

                    $error = $updatedShippingRateResponse->error;

                } else if (isset($updatedShippingRateResponse->errors)) {

                    $error = "Failed to update the Shipping address";

                }

                $this->helperObj->logMe("Failed error: " .
                    $error, $this->logFiles['log']['filename']);
            }
        }
        $subscriptionResponse = $this->modelObj->updateSubscription($subscriptionID, json_encode($updateRequest));

        if (isset($subscriptionResponse->error) || isset($subscriptionResponse->errors) ||
            empty($subscriptionResponse)) {

            $status = "failed";
            $error  = "Unknown error";
            if (isset($subscriptionResponse->error)) {

                $error = $subscriptionResponse->error;

            } else if (isset($subscriptionResponse->errors)) {

                $error = "Failed to update the subscription";

            }

            $this->helperObj->logMe("Failed to update the Checkout  Error: " .
                $error, $this->logFiles['log']['filename']);
            $response = array(
                "status"       => "failed",
                "message"      => "Failed to update the Checkout",
                "errorMessage" => $error,
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }

        $this->totalPrice = number_format((float)$this->totalPrice, 2, '.', '');
        $shippingPrice = number_format((float)$shippingPrice, 2, '.', '');
        $response         = array(
            "status"             => "success",
            "subscription_price" => $this->totalPrice,
            "melatonin_warning"  => $melatoninCount > 4 ? true : false,
            "shipping_amount"    => $shippingPrice,
            "shipping_rate_update_status" => trim($existingShippingPrice) != trim($shippingPrice) ? true : false,
            "errorMessage"       => "",
        );

        echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
        die();

    }

    /**
     * Retrive Checkout
     */
    public function RetrieveCheckout(Request $request) {

        $requestData = $_REQUEST['data'];

        $receivedData = json_decode($requestData);
        if (empty($receivedData->checkout->token)) {

            $response = array(
                "status"       => "Failed",
                "errorMessage" => "Missing required parameter Checkout token",
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();

        }

        $checkoutReturn = $this->modelObj->retrieveCheckout($receivedData->checkout->token, $receivedData->checkout->type);

        if (isset($checkoutReturn->error) || isset($checkoutReturn->errors) ||
            empty($checkoutReturn)) {

            $status = "failed";
            $error  = "Unknown error";
            if (isset($checkoutReturn->error)) {

                $error = $checkoutReturn->error;

            } else if (isset($checkoutReturn->errors)) {

                $error = $this->helperObj->convert_multi_array($checkoutReturn->errors);

            }

            $this->helperObj->logMe("Failed to load the Checkout for Token (" .
                $receivedData->checkout->token . "), Error: " .
                $error, $this->logFiles['log']['filename']);
            $response = array(
                "status"       => "failed",
                "errorMessage" => $error,
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die;
        }
        $objToArray = $this->helperObj->objectToArray($checkoutReturn);
        header('Content-Type: application/json');
        $json_string = json_encode($objToArray, JSON_PRETTY_PRINT);
        echo $_REQUEST['callback'] . '(' . $json_string . ')';
        die();
    }
    
    public function parseDom(){
    	$url="https://nurish-usd-dev.myshopify.com/pages/subscription-discount";
    
    	try{
        	$ch = curl_init();
        	curl_setopt($ch, CURLOPT_URL, $url);
        	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        	curl_setopt($ch,CURLOPT_USERAGENT,'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.13) Gecko/20080311 Firefox/2.0.0.13');
        	$html = curl_exec($ch);
        	curl_close($ch);
    	}catch(Exception $e){
    	    $this->helperObj->logMe("Failed to load the Checkout for Token (" .
                $receivedData->checkout->token . "), Error: " .
                $error, $this->logFiles['log']['filename']);
    		print_r($e->getMessage());
    	}
    
    	$dom = new \DOMDocument();
    	@$dom->loadHTML($html);
    	$xpath = new \DOMXPath($dom);
    	$inputNodes = $xpath->query('//input[@id="globalDiscountValue"]');
    
    	if(isset($inputNodes[0])){
    	    return $inputNodes[0]->getAttribute('value');
    	}
	
    }
    
    public function getOTPExtra($email){
    	$url="https://nurish-usd-dev.myshopify.com/pages/subscription-discount";
    
    	try{
        	$ch = curl_init();
        	curl_setopt($ch, CURLOPT_URL, $url);
        	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        	curl_setopt($ch,CURLOPT_USERAGENT,'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.13) Gecko/20080311 Firefox/2.0.0.13');
        	$html = curl_exec($ch);
        	curl_close($ch);
    	}catch(Exception $e){
    	    $this->helperObj->logMe("Failed to load the URL from CMS for -".$email, $this->logFiles['log']['filename']);
            //print_r($e->getMessage());
    	}
    
    	$dom = new \DOMDocument();
    	@$dom->loadHTML($html);
    	$xpath = new \DOMXPath($dom);
    	$inputNodes = $xpath->query('//input[@id="globalDiscountValue"]');
    
    	if(isset($inputNodes[0])){
    	    $discount = $this->getExtraOTPAmount($inputNodes[0]->getAttribute('value'));
    	    return $discount;
    	}
    	
    	$this->helperObj->logMe("Failed to load the json URL from CMS -".$email, $this->logFiles['log']['filename']);
    	return;
	
    }
    
    public function getExtraOTPAmount($url){
        // $url = '//cdn.shopify.com/s/files/1/0263/1293/3458/t/48/assets/subscription_schema.json?v=9998742680377307201';
    	if(!empty($url)){
    		$url  = 'https:'.$url;
    		$html = file_get_contents($url);
    		$discount = json_decode($html)->subscriptionDiscount;
    		if(trim($discount) != ''){
    		   if($discount > 0){
    		       return $discount; 
    		   }
    		}
    	}
    	$this->helperObj->logMe("Failed to load the discount from JSON file", $this->logFiles['log']['filename']);
    	return;
    }
    
}
