<?php

namespace App\Http\Controllers;
header("Access-Control-Allow-Origin: *");
use App\Model\IterableApi;
use App\CheckoutFunctions\checkoutFunctions;
use App\Helper\helper;
use Illuminate\Http\Request;
class IterableController extends Controller
{

    private $modelObj;
    private $checkObj;
    private $logDirectory;
    private $logFiles;
    private $logPath;
    private $helperObj;
    private $checkoutReturn;
    private $totalPrice = 0;
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->modelObj = new IterableApi();
        $this->checkObj = new checkoutFunctions();
        $this->helperObj = new Helper();

        $this->setPaths();
        $this->getLogFilesList();
        $this->setWithLogFiles();
    }
     public function setPaths() {

        $this->logDirectory = dirname(__FILE__) . "/../../../storage/logs/";
    }

    public function getLogFilesList() {

        $this->logPath = $this->logDirectory . date('dmY');

        if (!is_dir($this->logPath))
        {
         //Directory does not exist, so lets create it.
         mkdir($this->logPath);
         chmod($this->logPath, 0777);

        }

        $this->logFiles = array(
            'log' => array(
                "type" => "text",
                "filename" => $this->logPath . "/iterable-log.txt",
                "mode" => ""),
            );
    }

    public function setWithLogFiles() {

        if(!empty($this->logFiles)) {
            foreach ($this->logFiles as $fileDetails) {
                $fileName = $fileDetails['filename'];
                if($fileDetails['type'] == 'csv' && !file_exists($fileName)) {
                   $handler = fopen($fileName, $fileDetails['mode']);
                   fputcsv($handler, $fileDetails['columns']);
                   fclose($handler);
                   touch($fileName);
                   chmod($fileName, 0777);
                }
            }
        }
   }

    /**
     * Get user Iterable details
     */
    public function getUserIterableStatus() {

        $requestData = $_REQUEST['data'];

        $receivedData = json_decode($requestData);

        if(empty($receivedData->email)) {

            $response = array(
                "status" => "Failed",
                "errorMessage" => "Missing required parameters or the request is not correct",
            );

            echo $_REQUEST['callback'].'('.json_encode($response).')';
            die();
         }

         $subscriptionResponse = $this->modelObj->getIterableSubscription($receivedData->email);


         if (!empty($subscriptionResponse->msg) || empty($subscriptionResponse)) {

            $status = "failed";
            $error = "Unknown error";
            if (isset($subscriptionResponse->msg)) {

                $error = "Invalid Email";

            } else if (isset($subscriptionResponse->errors)) {

                $error = "Invalid Email";

            }

            $this->helperObj->logMe("Failed to get the Iterable details for Email (".
            $receivedData->email."), Error: " .
            $error, $this->logFiles['log']['filename']);
            $response = array(
                "status" => "failed",
                "message" => "Failed to get the Iterable details",
                "errorMessage" => $error,
            );

            echo $_REQUEST['callback'].'('.json_encode($response).')';
            die();
        }

        $responseData = array(
            'unsubscribedChannelIds' => $subscriptionResponse->user->dataFields->unsubscribedChannelIds,
            'subscribedMessageTypeIds' => $subscriptionResponse->user->dataFields->subscribedMessageTypeIds,
            'unsubscribedMessageTypeIds' => $subscriptionResponse->user->dataFields->unsubscribedMessageTypeIds

        );

        $response = array(
            "status" => "success",
            "response" => $responseData,
            "errorMessage" => "",
        );

        echo $_REQUEST['callback'].'('.json_encode($response).')';
        die();

    }

    /**
     * Update user Iterable details
     */
     public function updateIterableSubscription() {

        $requestData = $_REQUEST['data'];

        $receivedData = json_decode($requestData);

        if(empty($receivedData) || empty($receivedData->email)) {

            $response = array(
                "status" => "Failed",
                "errorMessage" => "Missing required parameters or the request is not correct",
            );

            echo $_REQUEST['callback'].'('.json_encode($response).')';
            die();
         }


         $updateRequest = array(
            "email" => $receivedData->email,
            "unsubscribedChannelIds" => 
            array_map('intval', $receivedData->unSubscribeChannelIds),
            "unsubscribedMessageTypeIds" => 
            array_map('intval', $receivedData->unSubscribePreferenceIds),
            "subscribedMessageTypeIds" => 
            array_map('intval', $receivedData->subscribePreferenceIds)
        );

        $subscriptionResponse = $this->modelObj->updateUserIterableSubscription(json_encode($updateRequest));

        if (!empty($subscriptionResponse->msg) || empty($subscriptionResponse)) {

            $status = "failed";
            $error = "Unknown error";
            if (isset($subscriptionResponse->msg)) {

                $error = "Invalid parameters";

            } else if (isset($subscriptionResponse->errors)) {

                $error = "Invalid parameters";

            }

            $this->helperObj->logMe("Failed to update the Iterable subscription 
            details for Email (".$receivedData->email."), Error: " .
            $error, $this->logFiles['log']['filename']);
            $response = array(
                "status" => "failed",
                "message" => "Failed to update the Iterable subscription",
                "errorMessage" => $error,
            );

            echo $_REQUEST['callback'].'('.json_encode($response).')';
            die();
        }

        $response = array(
            "status" => "success",
            "response" => 'updated',
            "errorMessage" => "",
        );

        echo $_REQUEST['callback'].'('.json_encode($response).')';
        die();

     }

    /**
      * add user to Iterable List
    */
    public function addUserToIterableList(){

        $requestData = $_REQUEST['data'];

        $receivedData = json_decode($requestData);

        $subscriptionRequest = array(

            "listId" => intval(env('ITERABLE_SMS_SUBSCRIBE_LIST_ID', true)),
            "subscribers" => array(
                array(
                    "email" => $receivedData->email,
                    "dataFields" => array(
                    "phoneNumber" => $receivedData->mobileNumber,
                    "First Name" => $receivedData->firstName,
                    "Last Name" => $receivedData->lastName,
                    "default_address" => array(
                        "zip" => $receivedData->postalCode
                    )
                  )
               )
            )

        );

        $createUserSubscribeListResponse = 
        $this->modelObj->addSubscriberToIterableList(json_encode($subscriptionRequest));

        if (!empty($createUserSubscribeListResponse->msg) || 
        empty($createUserSubscribeListResponse)) {

            $status = "failed";
            $error = "Unknown error";
            if (isset($subscriptionResponse->msg)) {

                $error = "Invalid parameters";

            } else if (isset($subscriptionResponse->errors)) {

                $error = "Invalid parameters";

            }

            $this->helperObj->logMe("Failed to update the 
            Iterable subscription details for Email (".$receivedData->email."), Error: " .
            $error, $this->logFiles['log']['filename']);
            $response = array(
                "status" => "failed",
                "message" => "Failed to update the Iterable subscription",
                "errorMessage" => $error,
            );

            echo $_REQUEST['callback'].'('.json_encode($response).')';
            die();
        }

        $response = array(
            "status" => "success",
            "response" => 'updated',
            "errorMessage" => "",
        );

        echo $_REQUEST['callback'].'('.json_encode($response).')';
        die();

      }

}
