<?php

namespace App\Http\Controllers;

header("Access-Control-Allow-Origin: *");
use App\Model\RechargeApi;
use App\Model\Shopify\Product;
use App\Model\GetPackCount;
use App\CheckoutFunctions\checkoutFunctions;
use App\Helper\helper;
use Illuminate\Http\Request;

class RechargeAddressController extends Controller
{

    private $modelObj;
    private $checkObj;
    private $packCount;
    private $logDirectory;
    private $logFiles;
    private $logPath;
    private $helperObj;
    private $checkoutReturn;
    private $productObj;

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct() {

        $this->modelObj   = new RechargeApi();
        $this->packCount  = new GetPackCount();
        $this->productObj = new Product();
        $this->checkObj   = new checkoutFunctions();
        $this->helperObj  = new Helper();

        $this->setPaths();
        $this->getLogFilesList();
        $this->setWithLogFiles();
    }


    public function setPaths() {

        $this->logDirectory = dirname(__FILE__) . "/../../../storage/logs/";
    }

    public function getLogFilesList() {

        $this->logPath = $this->logDirectory . date('dmY');

        if (!is_dir($this->logPath)) {
            //Directory does not exist, so lets create it.
            mkdir($this->logPath);
            chmod($this->logPath, 0777);

        }

        $this->logFiles = array(
            'log'             => array(
                "type"     => "text",
                "filename" => $this->logPath . "/log.txt",
                "mode"     => ""),
            'checkoutDetails' => array(
                "type"     => "csv",
                "filename" => $this->logDirectory . "checkoutOrderDetails.csv",
                "mode"     => "a",
                "columns"  => $this->checkObj->setCheckoutColumns()),
        );
    }

    public function setWithLogFiles() {

        if (!empty($this->logFiles)) {
            foreach ($this->logFiles as $fileDetails) {
                $fileName = $fileDetails['filename'];
                if ($fileDetails['type'] == 'csv' && !file_exists($fileName)) {
                    $handler = fopen($fileName, $fileDetails['mode']);
                    fputcsv($handler, $fileDetails['columns']);
                    fclose($handler);
                    touch($fileName);
                    //Make it writeable
                    chmod($fileName, 0777);
                }
            }

        }
    }

    /**
     * Update subscription shipping rate when updating the address
     */

    public function updateSubscriptionByAddressID() {

        if (empty($_REQUEST['data'])) {

            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo json_encode($response);
            die();

        }
        $requestData = $_REQUEST['data'];
        $receivedData = json_decode($requestData);

        if(empty($receivedData->customer_id) || empty($receivedData->address_id)) {

            $response = array(
                "status"       => "Failed",
                "errorMessage" => "The customerID or addressID is empty",
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        } 

        $customerSubscriptions = $this->modelObj->getAllSubscriptionsByCustomerID($receivedData->customer_id);

        if (isset($customerSubscriptions->error) || isset($customerSubscriptions->errors)) {
    
                $status = "failed";
                $error  = "Unknown error";
                if (isset($customerSubscriptions->error)) {
    
                    $error = $customerSubscriptions->error;
    
                } else if (isset($customerSubscriptions->errors)) {
    
                    $error = "Failed to get the subscriptions";
    
                }
    
                $this->helperObj->logMe("Failed error: " .
                    $error, $this->logFiles['log']['filename']);
        }

        if (empty($customerSubscriptions) || !isset($customerSubscriptions)) {

            $response = array(
                "status"       => "Success",
                "message"      => "There is no subscription in the given customer id",
                "errorMessage" => "",
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();

        }

        $usShippingRateAdjustableCountries = array(

            "alabama","arizona","arkansas","california","colorado","connecticut","delaware","florida","georgia",
            "idaho","illinois","indiana","iowa","kansas","kentucky","louisiana","maine","maryland","massachusetts",
            "michigan","minnesota","mississippi","missouri","montana","nebraska","nevada","new-hampshire",
            "new-jersey","new-mexico","new-york","north-carolina","north-dakota","ohio","oklahoma",
            "oregon","pennsylvania","rhode-island","south-carolina","south-dakota","tennessee","texas",
            "utah","vermont","virginia","washington","west-virginia","wisconsin","wyoming"

        );

        $usShippingRateNonAdjustableCountries = array(
            "alaska","hawaii"
        );

        $shopifyDomesticShippingPrice = '';
        $shopifyNonDomesticShippingPrice = '';

        $shippingZones = $this->productObj->getShopifyShippingZone();

        if (isset($shippingZones['shipping_zones'])) {

            foreach ($shippingZones['shipping_zones'] as $shipValue) {

                $shippingZoneName = strtolower(trim($shipValue['name']));
                if (strpos($shipValue['name'], ",") !== false ) {

                    $shippingZoneName = str_replace(" ", "", $shippingZoneName);
                    $shippingZoneName = str_replace(",", "-", $shippingZoneName);
                }

                if ($shippingZoneName == 'domestic' || $shippingZoneName == 'alaska-hawaii') {

                    if(!empty($shipValue['price_based_shipping_rates'])) {

                        foreach ($shipValue['price_based_shipping_rates'] as $priceValue) {

                            $shippingName = strtolower(trim($priceValue['name']));
                            $shippingName = str_replace(" ", "-", $shippingName);

                            if($shippingName == 'standard-shipping') {

                                if ($shippingZoneName == 'domestic') {

                                    $shopifyDomesticShippingPrice = $priceValue['price'];

                                } else {

                                    $shopifyNonDomesticShippingPrice = $priceValue['price'];

                                }
                            }
                        }
                    }  
                }
            }
        }

        $shippingRateUpdateStatus = 0;

        foreach ($customerSubscriptions->subscriptions as $subValue) {

            if (trim($subValue->address_id) == trim($receivedData->address_id)) {

                $subscriptionPrice = trim($subValue->price);

                $userAddressDetails = $this->modelObj->getAddressByAddressID($receivedData->address_id);

                if (!empty($userAddressDetails->address->province)){

                    $addressProvince = strtolower(trim($userAddressDetails->address->province));
                    $addressProvince = str_replace(" ", "-", $addressProvince);
        
                } else {
        
                    $addressProvince = '';
        
                }

                $shippingUpdateStatus = 0;
                $shippingPrice = '';
                $shippingCode = '';

                if (!empty($addressProvince) && in_array($addressProvince, $usShippingRateAdjustableCountries)) {

                    if ($subscriptionPrice > 20) {
                        $shippingPrice = 0;
                        $shippingCode = "Free Shipping";
                    } else {
                        $shippingPrice = !empty($shopifyDomesticShippingPrice) ? $shopifyDomesticShippingPrice : 8;
                        $shippingCode = "Standard Shipping";
                    }

                    $shippingUpdateStatus = 1;

                } elseif (!empty($addressProvince) && in_array($addressProvince, $usShippingRateNonAdjustableCountries)) {

                    $shippingPrice = !empty($shopifyNonDomesticShippingPrice) ? $shopifyNonDomesticShippingPrice : 8;
                    $shippingCode = "Standard Shipping";

                    $shippingUpdateStatus = 1;

                } else {

                    $shippingPrice = !empty($shopifyDomesticShippingPrice) ? $shopifyDomesticShippingPrice : 8;
                    $shippingCode = "Standard Shipping";

                    $shippingUpdateStatus = 1;

                }

                if ($shippingUpdateStatus == 1) {

                    $shippingUpdateRequest = array(

                        "shipping_lines_override" => array(
                        array(
                            "price" => $shippingPrice,
                            "code"  => $shippingCode,
                            "title" => "Standard Shipping"

                        )
                      )

                    );

                    $updatedShippingRateResponse = $this->modelObj->updateShippingRateByAddress(
                        $receivedData->address_id, json_encode($shippingUpdateRequest));

                        if (isset($updatedShippingRateResponse->error) || isset($updatedShippingRateResponse->errors) ||
                        empty($updatedShippingRateResponse)) {
            
                        $status = "failed";
                        $error  = "Unknown error";
                        if (isset($updatedShippingRateResponse->error)) {
            
                            $error = $updatedShippingRateResponse->error;
            
                        } else if (isset($updatedShippingRateResponse->errors)) {
            
                            $error = "Failed to update the Shipping address";
            
                        }
            
                        $this->helperObj->logMe("Failed error: " .
                            $error, $this->logFiles['log']['filename']);
                    }
                }

                $shippingRateUpdateStatus = 1;
            }
        }

        if($shippingRateUpdateStatus == 0) {

            $response         = array(
                "status"             => "success",
                "message"            => "There is no subscriptions in the given addressID",
                "errorMessage"       => "",
            );
    
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();

        }

        if ($shippingRateUpdateStatus == 1) {

            $response         = array(
                "status"             => "success",
                "message"            => "Successfully updated the subscriptions shipping amount of the given user",
                "errorMessage"       => "",
            );
    
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();

        }
    }
}

