<?php

namespace App\Http\Controllers;

header("Access-Control-Allow-Origin: *");
use App\Model\RechargeApi;
use App\Model\IterableApi;
use App\Helper\Helper;
use Illuminate\Http\Request;
use App\Model\Shopify\Product;


class SubscriptionController extends Controller
{
    private $rechargeObj;
    private $iterableObj;
    private $logDirectory;
    private $logFiles;
    private $logPath;
    private $helperObj;
    private $productObj;

    public function __construct() {

        $this->rechargeObj   = new RechargeApi();
        $this->iterableObj   = new IterableApi();
        $this->helperObj  = new Helper();
        $this->productObj = new Product();

        $this->setPaths();
        $this->getLogFilesList();
        $this->setWithLogFiles();
    }

    public function setPaths() {

        $this->logDirectory = dirname(__FILE__) . "/../../../storage/logs/";
    }

    public function getLogFilesList() {

        $this->logPath = $this->logDirectory . date('dmY');

        if (!is_dir($this->logPath)) {
            //Directory does not exist, so lets create it.
            mkdir($this->logPath);
            chmod($this->logPath, 0777);

        }

        $this->logFiles = array(
            'log'             => array(
                "type"     => "text",
                "filename" => $this->logPath . "/log-upsell.txt",
                "mode"     => ""),
            'activation'             => array(
                "type"     => "text",
                "filename" => $this->logPath . "/log.txt",
                "mode"     => ""),
            'subscriptionPauseLog'   => array(
                "type"     => "text",
                "filename" => $this->logPath . "/log-subscriptionPause.txt",
                "mode"     => ""),
            'otpToSubscriptionLog'   => array(
                "type"     => "text",
                "filename" => $this->logPath . "/log-otpToSubscription.txt",
                "mode"     => ""),
            'subscriptionReactivationWithCoupon'   => array(
                    "type"     => "text",
                    "filename" => $this->logPath . "/log-subscriptionReactivationWithCoupon.txt",
                    "mode"     => ""),
            'adjustNextChargeDate'   => array(
                "type"     => "text",
                "filename" => $this->logPath . "/log-adjustNextChargeDate.txt",
                "mode"     => "")
        );
    }

    public function setWithLogFiles() {

        if (!empty($this->logFiles)) {
            foreach ($this->logFiles as $fileDetails) {
                $fileName = $fileDetails['filename'];
                if ($fileDetails['type'] == 'csv' && !file_exists($fileName)) {
                    $handler = fopen($fileName, $fileDetails['mode']);
                    fputcsv($handler, $fileDetails['columns']);
                    fclose($handler);
                    touch($fileName);
                    //Make it writeable
                    chmod($fileName, 0777);
                }
            }

        }
    }

    /**
     * upsell endpoint to update subscription with addon
     */
    public function upsellSubscription() {

        if (empty($_REQUEST['data'])) {

            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo json_encode($response);
            die();
        }

        $requestData = $_REQUEST['data'];

        $receivedData = json_decode($requestData);

        if (empty($receivedData->subscription_id)) {


            $response = array(
                "status"       => "Failed",
                "errorMessage" => "The subscription id is missing in the request",
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }

        if(empty($receivedData->recharge_hash)) {

            $response = array(
                "status"       => "Failed",
                "errorMessage" => "The customer recharge hash is missing in the request",
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }

        $propertyStatus = 0;

        if (empty($receivedData->addons)) {

            $propertyStatus = 1;

        } else {

            foreach ($receivedData->addons as $Value) {

                if (empty($Value->product_id) || empty($Value->variant_id) ||
                empty($Value->title)) {

                    $propertyStatus = 1;
                }
            }
        }

        if ($propertyStatus == 1) {

            $response = array(
                "status"       => "Failed",
                "errorMessage" => "Missing required parameters in the line item properties",
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }

        $subscriptionID = trim($receivedData->subscription_id);
        $subscriptionDetails =  $this->rechargeObj->getSubscription($subscriptionID);

        if (!empty($subscriptionDetails->error) || !empty($subscriptionDetails->errors) ||
            empty($subscriptionDetails)) {

            $status = "failed";
            $error  = "Unknown error";
            if (!empty($subscriptionDetails->error)) {

                $error = $subscriptionDetails->error;

            } else if (!empty($subscriptionDetails->errors)) {

                $error = "Failed to get the subscription details";
            }
            $message = "Failed to upsell the subscription";
            $this->helperObj->logMe($error, $this->logFiles['log']['filename']);
            $this->sendResponse("Failed", $message, $error);
        }

        $customerID = $subscriptionDetails->subscription->customer_id;
        $customerDetails = $this->rechargeObj->getRechargeCustomerDetailsByID($customerID);

        if (!empty($customerDetails->error) || !empty($customerDetails->errors) ||
            empty($customerDetails)) {

            $status = "failed";
            $error  = "Unknown error";
            if (!empty($customerDetails->error)) {

                $error = $customerDetails->error;

            } else if (!empty($customerDetails->errors)) {

                $error = "Failed to get the customer details";
            }

            $message = "Failed to upsell the subscription";
            $this->helperObj->logMe($error, $this->logFiles['log']['filename']);
            $this->sendResponse("Failed", $message, $error);
        }

        $customerRechargeHash = $customerDetails->customer->hash;
	$customerEmail = $customerDetails->customer->email;
        $urlCustomerRechargeHash = trim($receivedData->recharge_hash);

        if (trim($customerRechargeHash) != trim($urlCustomerRechargeHash)) {

            $response = array(
                "status"       => "Failed",
                "errorMessage" => "Failed to validate the customer recharge hash",
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();

        } else {

            $endpointData = [];
            $vitaminCount = 1;
            $subscriptionDetails = $this->helperObj->objectToArray($subscriptionDetails);
            $shopifyProductDetails = $this->getShopifyProductDetails();

            foreach ($subscriptionDetails['subscription']['properties'] as $subProperty) {

                foreach ($shopifyProductDetails as $shValue) {

                    foreach ($shValue as $variantValue) {

                        $propertyName = strtolower(trim($subProperty['name']));
                        $propertyValue = trim($subProperty['value']);
                        if (strpos($propertyName, 'vitamin') !== false && strpos($propertyValue, '|') !== false) {
                            $freePill = false;
                            if (strpos($propertyValue, 'free_pill') !== false) {
                                $freePill = true;
                            }
                            $explodeValue = explode('|', $propertyValue);
                            if($explodeValue[3] == $variantValue['id'] || $explodeValue[4] == $variantValue['sku']) {

                                $endpointData[] = array(
                                    "name" => trim($subProperty['name']),
                                    "value" => trim($explodeValue[0]),
                                    "product_id" => intval($variantValue['product_id']),
                                    "variant_id" => intval($explodeValue[3]),
                                    "quantity" => intval($explodeValue[2]),
                                    "dosage" => trim($explodeValue[1]),
                                    "free_pill" => $freePill
                                );
                                $vitaminCount++;
                            }
                        }
                    }
                }
            }

            foreach ($receivedData->addons as $Value) {

                $endpointData[] = array(
                    "name" => 'Vitamin'. ($vitaminCount<10 ? '0'.$vitaminCount++ : $vitaminCount++),
                    "value" => trim($Value->title),
                    "product_id" => intval($Value->product_id),
                    "variant_id" => intval($Value->variant_id),
                    "quantity" => !empty($Value->quantity) ? intval($Value->quantity) : 1,
                    "dosage" => trim($Value->dosage),
                    "free_pill" => false
                );
            }

            $subscriptionUpdateRequest = array(
                "subscription_id" => $subscriptionID,
                "properties" => $endpointData
            );

            $updateResponse = $this->rechargeObj->updateUpsellSubscription(json_encode($subscriptionUpdateRequest));
            
	    if (strtolower($updateResponse['status']) == 'success') {

		    $response         = array(
		    "status"             => "success",
		    "subscription_price" => $updateResponse['subscription_price'],
		    "melatonin_warning"  => $updateResponse['melatonin_warning'] == 1 ? true : false,
		    "shipping_amount"    => $updateResponse['shipping_amount'],
		    "shipping_rate_update_status" => $updateResponse['shipping_rate_update_status'] == 1 ? true : false,
		    "errorMessage"       => "",
		);
	    } else {
		     $response = array(
		     "status"       => "Failed",
		     "errorMessage" => "Failed to update the subscription"."(".$subscriptionID.") - ".$customerEmail,
		    );

		    $error = "Failed to update the subscription"."(".$subscriptionID.") - ".$customerEmail;
		    $this->helperObj->logMe($error, $this->logFiles['log']['filename']);

	    }

	    echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
	    die();

        }
    }


	public function reactivateSubscriptionByEmail() {

        if(!isset($_REQUEST['email'])) {

            $message = "Failed to reactivate the subscription";
            $error = "Missing required parameter email";
            $this->sendResponse("Failed", $message, $error);
        }
        $email = $_REQUEST['email'];
        $customer = $this->rechargeObj->getRechargeCustomerDetailsByEmail($email);
        if (isset($customer->error) || isset($customer->errors) ||
        empty($customer)) {

            $status = "failed";
            $error  = "Unknown error";
            if (isset($customer->error)) {

                $error = $customer->error;

            } else if (isset($customer->errors)) {

                $errorToArray = $this->helperObj->objectToArray($customer->errors);
                $error        = implode(", ",$errorToArray);

            }
            $message = "Failed to reactivate the subscription";
            $this->helperObj->logMe($error, $this->logFiles['log']['reactivate_filename']);
            $this->sendResponse("Failed", $message, $error);
        }
        if(!isset($customer->customers[0]->id)) {

            $message = "Failed to reactivate the subscription";
            $error = "Customer not found";
            $this->sendResponse("Failed", $message, $error);
        }
        foreach($customer->customers as $customerdet) {
            if($customerdet->email == $email) {
                $customer = $customerdet;
            }
        }
        $customer_id = $customer->id;
        $subscription = $this->rechargeObj->getSubscriptionByCustomerID($customer_id);
        if (isset($subscriptions->error) || isset($subscription->errors) ||
        empty($subscription)) {

            $status = "failed";
            $error  = "Unknown error";
            if (isset($subscription->error)) {

                $error = $subscription->error;

            } else if (isset($subscription->errors)) {

                $errorToArray = $this->helperObj->objectToArray($subscription->errors);
                $error        = implode(", ",$errorToArray);

            }
            $message = "Failed to reactivate the subscription";
            $this->helperObj->logMe($error, $this->logFiles['log']['reactivate_filename']);
            $this->sendResponse("Failed", $message, $error);
        }
        if(empty($subscription->subscriptions)) {

            $message = "Failed to reactivate the subscription";
            $error = "Subscription not found";
            $this->sendResponse("Failed", $message, $error);
        }
        $subscription_id = $subscription->subscriptions[0]->id;
        $res = $this->rechargeObj->activateSubscription($subscription_id);

        if (isset($res->error) || isset($res->errors) ||
        empty($res)) {

            $status = "failed";
            $error  = "Unknown error";
            if (isset($res->error)) {

                $error = $res->error;

            } else if (isset($res->errors)) {

                $errorToArray = $this->helperObj->objectToArray($res->errors);
                $error        = implode(", ",$errorToArray);

            }
            $message = "Failed to reactivate the subscription";
            $this->helperObj->logMe($error, $this->logFiles['log']['reactivate_filename']);
            $this->sendResponse("Failed", $message, $error);
        }
        else{
            $this->sendResponse("Success", "Subscription reactivated successfully", "");
        }
    }

    /**
     * Reactivate cancelled recharge subscription
    */
    public function reactivateSubscriptionById() {

        if (empty($_REQUEST['data'])) {

            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo json_encode($response);
            die();

        }

        $requestData = $_REQUEST['data'];

        $receivedData = json_decode($requestData);

        $subscriptionIdStatus = 0;
        $subscription_id = '';
        $subscriptionIDs = array();

        foreach ($receivedData->subscriptions as $dataValue) {

          if (empty($dataValue->subscription_id)) {
            $subscriptionIdStatus = 1;
          } else {
              if (empty($subscription_id)) {
                $subscription_id = $dataValue->subscription_id;
              }
            array_push($subscriptionIDs,$dataValue->subscription_id);
          }
        }

        if ($subscriptionIdStatus == 1 || empty($receivedData->subscriptions)) {

            $message = "Failed to reactivate the subscription";
            $error = "Missing required parameter subscription_id";
            $this->sendResponse("Failed", $message, $error);
        }

        if (empty($receivedData->recharge_hash)) {

            $message = "Failed to reactivate the subscription";
            $error = "Missing required parameter recharge customer hash";
            $this->sendResponse("Failed", $message, $error);
        }

        $subscription_id = trim($subscription_id);
        $recharge_hash = trim($receivedData->recharge_hash);

        $subscriptionDetails = $this->rechargeObj->getSubscription($subscription_id);

        if (!isset($subscriptionDetails) || isset($subscriptionDetails->error)
         || isset($subscriptionDetails->errors) || empty($subscriptionDetails->subscription->customer_id)) {

            $status = "failed";
            $error  = "Unknown error";
            if (isset($subscriptionDetails->error)) {

                $error = $subscriptionDetails->error;

            } else if (isset($subscriptionDetails->errors)) {

                $error = $subscriptionDetails->errors;
            }

            $message = "Failed to get the subscription details";
            $this->helperObj->logMe($error, $this->logFiles['activation']['filename']);
            $this->sendResponse("Failed", $message, $error);

        }

        $customerDetails = $this->rechargeObj->getRechargeCustomerDetailsByID($subscriptionDetails->subscription->customer_id);

        if (!isset($customerDetails) || isset($customerDetails->error)
         || isset($customerDetails->errors) || empty($customerDetails->customer->hash)) {

            $status = "failed";
            $error  = "Unknown error";
            if (isset($customerDetails->error)) {

                $error = $customerDetails->error;

            } else if (isset($customerDetails->errors)) {

                $error = $customerDetails->errors;
            }

            $message = "Failed to get the customer details";
            $this->helperObj->logMe($error, $this->logFiles['activation']['filename']);
            $this->sendResponse("Failed", $message, $error);

        }

        $rechargeCustomerHash = trim($customerDetails->customer->hash);

        if ($recharge_hash == $rechargeCustomerHash) {

            $subscriptionIDs = array_unique($subscriptionIDs);
            $userSubscriptionIDs = implode( ",", $subscriptionIDs);

            $iterableRequest = array(
                "email" => $customerDetails->customer->email,
                "dataFields" => array(
                    "subscriptionID" => $userSubscriptionIDs,
                    "rechargeHash" => $recharge_hash
                )
            );

            $errorResponses = array();
            $errorStatus = 0;

            foreach ($subscriptionIDs as $subscriptionID) {

                $reactivateResponse = $this->rechargeObj->activateSubscription($subscriptionID);

                if (isset($reactivateResponse->error) || isset($reactivateResponse->errors) ||
            empty($reactivateResponse)) {

                $errorStatus = 1;

                $error  = "Unknown error";
                if (isset($reactivateResponse->error)) {

                    $error  = $reactivateResponse->error;
                    array_push($errorResponses, $subscriptionID.":".$error);

                } else if (isset($reactivateResponse->errors)) {

                    $error = $reactivateResponse->errors;

                    $error = !empty($error) ? $error : "Iterable user-data update request is not correct";

                    array_push($errorResponses, $subscriptionID.":".$error);

                }


              }

            }

            if ($errorStatus == 1) {

                $subscriptionErrors = implode( ",", $errorResponses);

                $error = "Failed subscriptions - ".$subscriptionErrors;

                $message = "Failed to reactivate the subscription";
                $this->helperObj->logMe($error, $this->logFiles['activation']['filename']);
                $this->sendResponse("Failed", $message, $error);
            }
            else{

                $updateIterableData = $this->iterableObj->updateUserDataFields(json_encode($iterableRequest));

                $this->sendResponse("Success", "Subscription reactivated successfully", "");
            }

        } else {
            $message = "Failed to reactivate the subscription";
            $error = "Customer hash tags are not matched";
            $this->helperObj->logMe($error, $this->logFiles['activation']['filename']);
            $this->sendResponse("Failed", $message, $error);
        }
    }


    public function sendResponse($status,$message, $error) {
        $response = array(
            "status"       => $status,
            "message" => $message,
            "error" => $error
        );
        echo json_encode($response);
        die();

    }

    /**
     * Function to get all shopify products
    */
    function getShopifyProductDetails(){

        $shopifyProductDetails = array();
        $pageNo = 1;
        $limit  = 100;

        do {
            try{
                //echo PHP_EOL."Shopify product page count - " . $pageNo;

                if(!empty($products['link'])){

                    $products = $this->productObj->getShopifyProducts($limit,$products['link']);

                 } else {

                    $products = $this->productObj->getShopifyProducts(
                        $limit);
                }
            }
            catch (Exception $e){

                echo PHP_EOL."Error form shopify when fetching the
                            product details and error is -  ".$e->getMessage();
                die;
            }

            if(!empty($products['error'])) {

                $response = array(
                    "status"       => "Failed",
                    "errorMessage" => $products['error'],
                );

                $error = "Error - ".$products['error'].", Shopify API name:p80w-upsell";
                $this->helperObj->logMe($error, $this->logFiles['log']['filename']);

                echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                die();
            }

            if ($products){

                foreach ($products['body'] as $product){
                    $shopifyProductDetails[] = $product['variants'];
                }
                $productCount = count($products['body']);

            }
            $pageNo++;

            if (empty($products['link'])) {
                break;
            }
        }
        while($productCount == $limit);

        return $shopifyProductDetails;
    }

    /**
     * Pause(change charge date) a subscription and create 'Subscription pause' event
     */
    function pauseSubscription(request $data) {

        if (empty($data['data'])) {

            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $this->requestResponse($status, $message, $error, $data->callback);
        }
        $receivedData = json_decode($data['data']);
        if (empty($receivedData)) {
            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $this->requestResponse($status, $message, $error, $data->callback);
        }
        $emptyKey = null;
        $requiredRequestKeys = ["subscription_id", "paused_period", "hash_value"];
        $requestKeys = [];
        $valueStatus = 0;
        foreach ($receivedData as $key => $receiveValue) {

            $requestKeys[] = $key;
            if (empty($receiveValue) && $valueStatus == 0) {
                $emptyKey = $key;
                $valueStatus = 1;
            }
        }
        $keyCompareStatus = array_diff($requiredRequestKeys, $requestKeys);
        if(!empty($keyCompareStatus)) {

            $missedParameters = implode(",",$keyCompareStatus);
            $message = $missedParameters." endpoint parameters are missing";
            $error = "Some parameters are missing in the endpoint request";
            $status = "Failed";
            $this->requestResponse($status, $message, $error, $data->callback);
        }
        if (!empty($emptyKey)) {

            $message = $emptyKey." is missing";
            $error = "Bad request";
            $status = "Failed";
            $this->requestResponse($status, $message, $error, $data->callback);
        }

        $subscriptionID = trim($receivedData->subscription_id);
        $subscriptionDetails =  $this->rechargeObj->getSubscription($subscriptionID);

        if (!empty($subscriptionDetails->error) || !empty($subscriptionDetails->errors) ||
            empty($subscriptionDetails)) {

            $status = "failed";
            $error  = "Unknown error";
            if (!empty($subscriptionDetails->error)) {

                $error = $subscriptionDetails->error;

            } else if (!empty($subscriptionDetails->errors)) {

                $error = "Failed to get the subscription details by using SubscriptionID:".$subscriptionID;
            }
            $message = "Failed to get the subscription details from recharge";
            $this->helperObj->logMe($error, $this->logFiles['subscriptionPauseLog']['filename']);
            $this->requestResponse("Failed", $message, $error, $data->callback);
        }

        $customerID = $subscriptionDetails->subscription->customer_id;
        $customerDetails = $this->rechargeObj->getRechargeCustomerDetailsByID($customerID);

        if (!empty($customerDetails->error) || !empty($customerDetails->errors) ||
            empty($customerDetails)) {

            $status = "failed";
            $error  = "Unknown error";
            if (!empty($customerDetails->error)) {

                $error = $customerDetails->error;

            } else if (!empty($customerDetails->errors)) {

                $error = "Failed to get the customer details by using CustomerID:".$customerID;
            }

            $message = "Failed to upsell the customer details from recharge";
            $this->helperObj->logMe($error, $this->logFiles['subscriptionPauseLog']['filename']);
            $this->requestResponse("Failed", $message, $error, $data->callback);
        }

        $customerHash = strtolower(trim($receivedData->hash_value));
        $originalCustomerHash = strtolower(trim($customerDetails->customer->hash));

        if ($customerHash != $originalCustomerHash) {

            $message = "Customer hash values are not matching";
            $error = "Security breach";
            $status = "Failed";
            $this->requestResponse($status, $message, $error, $data->callback);
        }
        if (empty($subscriptionDetails->subscription->next_charge_scheduled_at)) {

            $message = "The next charge date is not found in the subscription - ".$subscriptionID;
            $error = "Next charge date not found";
            $status = "Failed";
            $this->requestResponse($status, $message, $error, $data->callback);
        }

        $subscriptionCurrentChargeDate = $subscriptionDetails->subscription->next_charge_scheduled_at;
        $subscriptionCurrentChargeDate = date('Y-m-d', strtotime($subscriptionCurrentChargeDate));
        $pausePeriod = (int)$receivedData->paused_period;
        $newChargeDate = date('Y-m-d',strtotime('+'.$pausePeriod.' days',strtotime($subscriptionCurrentChargeDate)));
        $payload = array(
            "date" => $newChargeDate
        );
        $subscriptionUpdateResponse = $this->rechargeObj->updateSubscriptionChargeDate($subscriptionID, json_encode($payload));

        if (!empty($subscriptionUpdateResponse->error) || !empty($subscriptionUpdateResponse->errors) ||
        empty($subscriptionUpdateResponse)) {

            $status = "failed";
            $error  = "Unknown error";
            if (!empty($subscriptionUpdateResponse->error)) {

                $error = $subscriptionUpdateResponse->error;

            } else if (!empty($subscriptionUpdateResponse->errors)) {

                $error = "Failed to pause the subscription - ".$subscriptionID;
            }

            $message = "Failed to pause the subscription in recharge";
            $this->helperObj->logMe($error, $this->logFiles['subscriptionPauseLog']['filename']);
            $this->requestResponse("Failed", $message, $error, $data->callback);
        }
        $eventRequest = array(
            "events" => array(
            array("email" => strtolower($customerDetails->customer->email),
            "eventName" => "Subscription paused",
            "dataFields" => array("delay" => trim($receivedData->paused_period)))
            )
        );
        $eventResponse = $this->iterableObj->createNewIterableEvent(json_encode($eventRequest));

        if(isset($eventResponse->successCount) || $eventResponse->successCount > 0) {

            $this->helperObj->logMe("Paused subscription iterable event - Successfully created event for Email (".
            strtolower($customerDetails->customer->email).")", $this->logFiles['subscriptionPauseLog']['filename']);
        } else {
            $this->helperObj->logMe("Cancelled subscription iterable event - Failed to create event for Email (".
            strtolower($customerDetails->customer->email).")", $this->logFiles['subscriptionPauseLog']['filename']);
        }

        $message = "Successfully paused the subscription";
        $error = "";
        $status = "Success";
        $this->requestResponse($status, $message, $error, $data->callback);
    }

    public function requestResponse($status, $message, $error, $callback) {

        $response = array(
            "status"       => $status,
            "message" => $message,
            "error" => $error
        );
        echo $callback . '(' . json_encode($response) . ')';
        die();

    }

    /**
     * Convert OTP to subscription
     */
    public function otpToSubscription(request $data) {

        if (empty($data['data'])) {

            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $this->requestResponse($status, $message, $error, $data->callback);
        }
        $receivedData = json_decode($data['data']);
        if (empty($receivedData)) {
            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $this->requestResponse($status, $message, $error, $data->callback);
        }
        $emptyKey = null;
        $requiredRequestKeys = ["address_id", "hash_value"];
        $requestKeys = [];
        $valueStatus = 0;
        foreach ($receivedData as $key => $receiveValue) {

            $requestKeys[] = $key;
            if (empty($receiveValue) && $valueStatus == 0) {
                $emptyKey = $key;
                $valueStatus = 1;
            }
        }
        $keyCompareStatus = array_diff($requiredRequestKeys, $requestKeys);
        if(!empty($keyCompareStatus)) {

            $missedParameters = implode(",",$keyCompareStatus);
            $message = $missedParameters." endpoint parameters are missing";
            $error = "Some parameters are missing in the endpoint request";
            $status = "Failed";
            $this->requestResponse($status, $message, $error, $data->callback);
        }
        if (!empty($emptyKey)) {

            $message = $emptyKey." is missing";
            $error = "Bad request";
            $status = "Failed";
            $this->requestResponse($status, $message, $error, $data->callback);
        }
        $lineItemParameterStatus = 1;
        if (empty($receivedData->line_items)) {

            $lineItemParameterStatus = 0;
        } else {

            foreach ($receivedData->line_items as $key => $receiveValue) {

                if (empty($receiveValue->variant_id) || empty($receiveValue)) {
                    $lineItemParameterStatus = 0;
                }
            }
        }
        $discountCode = null;
        $addressID = trim($receivedData->address_id);
        $otpChargeDetails = $this->rechargeObj->getOtpChargeDetails($addressID);
        if (!empty($otpChargeDetails->error) || !empty($otpChargeDetails->errors) ||
            empty($otpChargeDetails)) {

            $status = "failed";
            $error  = "Unknown error";
            if (!empty($otpChargeDetails->error)) {

                $error = $otpChargeDetails->error;

            } else if (!empty($otpChargeDetails->errors)) {

                $error = "Failed to get the charge details by using addressID:".$addressID;
            }

            $message = "Failed to get the charge details from recharge";
            $this->helperObj->logMe($error, $this->logFiles['otpToSubscriptionLog']['filename']);
            $this->requestResponse("Failed", $message, $error, $data->callback);
        }
        $customerHash = strtolower(trim($receivedData->hash_value));
        $originalCustomerHash = null;
        $customerEmail = null;
        $username = null;
        $rechargeChargeDetails = null;
        $subscriptionExistingStatus = null;
        foreach ($otpChargeDetails->charges as $chargeValue) {

            if ($addressID == trim($chargeValue->address_id) &&
            $customerHash == strtolower(trim($chargeValue->customer_hash))) {

                $originalCustomerHash = $chargeValue->customer_hash;
                $customerEmail = $chargeValue->email;
                $username  = !empty($chargeValue->first_name) ? $chargeValue->first_name.' ' : '';
                $username .= !empty($chargeValue->last_name) ? $chargeValue->last_name : '';
                $chargeTags = strtolower(trim($chargeValue->tags));
                if (strpos($chargeTags, 'otp' ) !== false) {
                    $rechargeChargeDetails = $chargeValue;
                }
                if (strpos($chargeTags, 'subscription' ) !== false &&
                strtolower(trim($chargeValue->status)) == 'queued' ) {
                    $subscriptionExistingStatus = $chargeValue->status;
                }
            }
        }
        if (empty($originalCustomerHash)) {

            $message = "Customer hash value is not matching";
            $error = "Security breach";
            $status = "Failed";
            $this->requestResponse($status, $message, $error, $data->callback);
        }
        if (!empty($subscriptionExistingStatus)) {

            $message = "There is already a queued charge in the given addressID:".$addressID;
            $error = "Existing an active queued charge";
            $status = "Failed";
            $this->requestResponse($status, $message, $error, $data->callback);
        }
        $chargeTags = strtolower(trim($otpChargeDetails->charges[0]->tags));
        if (empty($rechargeChargeDetails)) {
            $message = "The charge in the given addressID is not an OTP charge";
            $error = "OTP charge not found";
            $status = "Failed";
            $this->requestResponse($status, $message, $error, $data->callback);
        }
        $otpLineItemQuantity = [];
        $lineItemProperties = [];
        $chargeVariantIDs = [];
        $otpFreePill = [];
        foreach ($rechargeChargeDetails->line_items[0]->properties as $propertyValue) {

            if (!empty($propertyValue->name) && $propertyValue->name != "one_time_purchase") {
                $lineItemProperties[] = $propertyValue;
                $propertyKey = strtolower(trim($propertyValue->name));
                if (strpos($propertyKey, 'vitamin') !== false) {
                    $explodeValue = explode("|", $propertyValue->value);
                    $otpLineItemQuantity[$explodeValue[3]] = $explodeValue[2];
                    $chargeVariantIDs[] = $explodeValue[3];
                    if (strpos($propertyValue->value, "free_pill") !== false) {
                        $otpFreePill[] = $explodeValue[3];
                    }
                }
            }
        }
        if (empty($chargeVariantIDs)) {

            $message = "The line items in the OTP charge is empty addressID:".$addressID;
            $error = "Line items not found in the OTP charge";
            $status = "Failed";
            $this->requestResponse($status, $message, $error, $data->callback);
        }
        if ($lineItemParameterStatus == 1) {

            $endpointVariantIDMismatchStatus = null;
            foreach ($receivedData->line_items as $lineValue) {

                if (in_array($lineValue->variant_id, $chargeVariantIDs) == false) {
                    $endpointVariantIDMismatchStatus = 1;
                }
                if (count($receivedData->line_items) != count($chargeVariantIDs)) {
                    $endpointVariantIDMismatchStatus = 1;
                }
            }
            if ($endpointVariantIDMismatchStatus == 1) {

                $message = "The requested endpoint variantID and OTP charge variantID are mismatched";
                $error = "Product mismatch found";
                $status = "Failed";
                $this->requestResponse($status, $message, $error, $data->callback);
            }
        }
        $lineItemProperties[] = (object)array(
            "name" => "charge_interval_frequency",
            "value" => 28
        );
        $lineItemProperties[] = (object)array(
            "name" => "shipping_interval_frequency",
            "value" => 28
        );
        $lineItemProperties[] = (object)array(
            "name" => "shipping_interval_unit_type",
            "value" => "day"
        );
        $productVariantIDs = null;
        if ($lineItemParameterStatus == 1) {

            $productVariantIDs = $receivedData->line_items;
        } else {

            foreach ($chargeVariantIDs as $chargeValue) {

                $productVariantIDs[] = (object) ["variant_id" => intval($chargeValue)];
            }
        }
        $variantErrorStatus = 0;
        $subscriptionPrice = null;
        foreach ($productVariantIDs as $lineValue) {

            $variantDetails = $this->productObj->getShopifyVariantByID($lineValue->variant_id);
            if (!empty($variantDetails['errors']) || !empty($variantDetails['error'])) {

                $variantErrorStatus = 1;
                $variantMismatchID = $lineValue->variant_id;
                $variantError = !empty($variantDetails['errors']) ? $variantDetails['errors'] :
                $variantDetails['error'];
                break;
            }
            foreach ($otpLineItemQuantity as $propertyKey => $otpPropertyValue) {
                if (in_array($propertyKey, $otpFreePill)) {
                    continue;
                }
                if (trim($propertyKey) == trim($variantDetails['variant']['id'])) {
                    $subscriptionPrice = floatval($subscriptionPrice) +
                    (floatval($variantDetails['variant']['price']) * intval($otpPropertyValue));
                }
            }
        }
        if ($variantErrorStatus == 1) {

            $message = "The variantID in the endpoint request is not valid variantID: ".$variantMismatchID.", Error: ".$variantError;
            $error = "Invalid variantID found";
            $status = "Failed";
            $this->requestResponse($status, $message, $error, $data->callback);
        }
        $subscriptionSKU = "vitamin-pack";
        $today = time();
        $todayDate = date('Y-m-d');
        $defaultSubscriptionPeriod = 28;
        $otpChargeProcessedDate = date('Y-m-d', strtotime($rechargeChargeDetails->processed_at));
        $otpChargeProcessedDate = strtotime($otpChargeProcessedDate);
        $dateDiff = $today - $otpChargeProcessedDate;
        $dateDiffDays = round($dateDiff / (60 * 60 * 24));
        $nextChargePeriod = $defaultSubscriptionPeriod - $dateDiffDays;
        if ($nextChargePeriod <= 0) {
            $nextChargeDate = $todayDate;
        } else {
            $nextChargeDate = date('Y-m-d',strtotime('+'.$nextChargePeriod.' days',strtotime($todayDate)));
        }
        $shopifyVariantId = 31228232728658;
        $intervalUnit = "day";

        $subscriptionRequest = array(
            "address_id" => $rechargeChargeDetails->address_id,
            "next_charge_scheduled_at" => $nextChargeDate,
            //"next_charge_scheduled_at" => $todayDate,
            "quantity" => !empty($rechargeChargeDetails->line_items[0]->quantity) ?
            $rechargeChargeDetails->line_items[0]->quantity : 1,
            "price" => floatval($subscriptionPrice),
            "sku" => $subscriptionSKU,
            "shopify_variant_id" => $shopifyVariantId,
            "order_interval_unit" => $intervalUnit,
            "order_interval_frequency" => $defaultSubscriptionPeriod,
            "charge_interval_frequency" => $defaultSubscriptionPeriod,
            "properties" => $lineItemProperties
        );
        $subscriptionCreateResponse = $this->rechargeObj->createSubscription(json_encode($subscriptionRequest));
        if (!empty($subscriptionCreateResponse->error) || !empty($subscriptionCreateResponse->errors) ||
            empty($subscriptionCreateResponse)) {

            $status = "failed";
            $error  = "Unknown error";
            if (!empty($subscriptionCreateResponse->error)) {

                $error = $subscriptionCreateResponse->error;

            } else if (!empty($subscriptionCreateResponse->errors)) {

                $error = "Failed to create the subscription under the addressID:".$addressID;
            }
            $message = "Failed to create the subscription in recharge";
            $this->helperObj->logMe($error, $this->logFiles['otpToSubscriptionLog']['filename']);
            $this->requestResponse("Failed", $message, $error, $data->callback);
        }
        // Create iterable event convertToSubscription
        $eventRequest = array(
            "events" => array(
            array("email" => strtolower($customerEmail),
            "eventName" => "convertToSubscription",
            "dataFields" => array("userName" => $username))
            )
        );
        $eventResponse = $this->iterableObj->createNewIterableEvent(json_encode($eventRequest));

        if(isset($eventResponse->successCount) || $eventResponse->successCount > 0) {

            $this->helperObj->logMe("convertToSubscription iterable event - Successfully created event for Email (".
            strtolower($customerEmail).")", $this->logFiles['otpToSubscriptionLog']['filename']);
        } else {
            $this->helperObj->logMe("convertToSubscription iterable event - Failed to create event for Email (".
            strtolower($customerEmail).")", $this->logFiles['otpToSubscriptionLog']['filename']);
        }
        // Applying the discount to the created subscription charge
        $subscriptionID = $subscriptionCreateResponse->subscription->id;
        $subscriptionChargeDetails = $this->rechargeObj->getChargeDetailsBySubscriptionID($subscriptionID);
        if (!empty($subscriptionChargeDetails->error) || !empty($subscriptionChargeDetails->errors) ||
            empty($subscriptionChargeDetails)) {

            $status = "failed";
            $error  = "Unknown error";
            if (!empty($subscriptionChargeDetails->error)) {

                $error = $subscriptionChargeDetails->error;

            } else if (!empty($subscriptionChargeDetails->errors)) {

                $error = "Failed to get the charge details by using subscriptionID: ".$subscriptionID;
            }

            $message = "Failed to get the subscription charge details from recharge";
            $this->helperObj->logMe($error, $this->logFiles['otpToSubscriptionLog']['filename']);
            $this->requestResponse("Failed", $message, $error, $data->callback);
        }
        $subscriptionChargeStatus = null;
        $subscriptionchargeID = null;
        foreach ($subscriptionChargeDetails->charges as $chargeValue) {

            $subscriptionChargeTags = strtolower(trim($chargeValue->tags));
            if (strpos($subscriptionChargeTags, 'subscription' ) !== false &&
            strtolower(trim($chargeValue->status)) == 'queued' ) {

                $subscriptionchargeID = $chargeValue->id;
                $subscriptionChargeStatus = $chargeValue->status;
            } 
        }
        if (empty($subscriptionChargeStatus)) {

            $message = "The subscription queued charge not found for apply the discount subscriptionID: "
            .$subscriptionID;
            $error = "Queued charge not found";
            $status = "Failed";
            $this->requestResponse($status, $message, $error, $data->callback);
        }
        if (!empty($receivedData->discountCode)) {

            $discountCode = trim($receivedData->discountCode);
            $discountCodeDetails = $this->rechargeObj->getDiscountDetailsByDiscountCode($discountCode);
            if (!empty($discountCodeDetails->error) || !empty($discountCodeDetails->errors) ||
            empty($discountCodeDetails)) {

                $status = "failed";
                $error  = "Unknown error";
                if (!empty($discountCodeDetails->error)) {

                    $error = $discountCodeDetails->error;

                } else if (!empty($discountCodeDetails->errors)) {

                    $error = "Failed to get the discount code details from recharge";
                }

                $message = "Failed to get the discount details from recharge by using the provided discountCode: "
                .$discountCode;
                $this->helperObj->logMe($error, $this->logFiles['otpToSubscriptionLog']['filename']);
                $this->requestResponse("Failed", $message, $error, $data->callback);
            }
            $discountRequest = array(
                "discount_code" => $discountCode
            );
            $discountApplyResponse = $this->rechargeObj->applyDiscountToChargeByDiscountCode($subscriptionchargeID, 
            json_encode($discountRequest));
            if (!empty($discountApplyResponse->error) || !empty($discountApplyResponse->errors) ||
            empty($discountApplyResponse)) {

                $status = "failed";
                $error  = "Unknown error";
                if (!empty($discountApplyResponse->error)) {

                    $error = $discountApplyResponse->error;

                } else if (!empty($discountApplyResponse->errors)) {

                    $error = "Failed to apply the discount to the charge";
                }

                $message = "Failed to apply the discount to the charge chargeID: ".$subscriptionchargeID;
                $this->helperObj->logMe($error, $this->logFiles['otpToSubscriptionLog']['filename']);
                $this->requestResponse("Failed", $message, $error, $data->callback);
            }
            $message = "Successfully created the subscription and applied the discount";
            $error = "";
            $status = "Success";
            $this->requestResponse($status, $message, $error, $data->callback);
        }
        $message = "Successfully created the subscription";
        $error = "";
        $status = "Success";
        $this->requestResponse($status, $message, $error, $data->callback);
    }
    
    /**
     * Reactivate cancelled recharge subscription with coupon
    */
    public function reactivateSubscriptionWithCoupon() {

        if (empty($_REQUEST['data'])) {

            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
            
        }

        $requestData = $_REQUEST['data'];

        $receivedData = json_decode($requestData);


        $subscriptionId = '';
        $customerHash = '';


        if (!isset($receivedData->subscriptionId) || empty($receivedData->subscriptionId)) {
            
            $message = "Failed to reactivate the subscription";
            $error = "Missing required parameter subscription_id";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
                
        }

        if (!isset($receivedData->customerHash) || empty($receivedData->customerHash)) {

            $message = "Failed to reactivate the subscription";
            $error = "Missing required parameter recharge customer hash";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
            
        }

        $subscriptionId = trim($receivedData->subscriptionId);
        $customerHash = trim($receivedData->customerHash);

        $subscriptionDetails = $this->rechargeObj->getSubscription($subscriptionId);

        if (!isset($subscriptionDetails) || isset($subscriptionDetails->error)
         || isset($subscriptionDetails->errors) || empty($subscriptionDetails->subscription->customer_id)) {

            $status = "Failed";
            $error  = "Unknown error";
            if (isset($subscriptionDetails->error)) {

                $error = $subscriptionDetails->error;

            } else if (isset($subscriptionDetails->errors)) {

                $error = $subscriptionDetails->errors;
            }
            $message = "Failed to get the subscription details";

            $this->helperObj->logMe(
                "Failed to get the subscription details: ".
                " | customerHash -> ".$customerHash.
                " | Subscription -> ".$subscriptionId.
                " | Error -> ".$error,
                $this->logFiles['subscriptionReactivationWithCoupon']['filename']
            );
            
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
            
        }

        $customerDetails = $this->rechargeObj->getRechargeCustomerDetailsByID($subscriptionDetails->subscription->customer_id);

        if (!isset($customerDetails) || isset($customerDetails->error)
         || isset($customerDetails->errors) || empty($customerDetails->customer->hash)) {

            $status = "Failed";
            $error  = "Unknown error";
            if (isset($customerDetails->error)) {

                $error = $customerDetails->error;

            } else if (isset($customerDetails->errors)) {

                $error = $customerDetails->errors;
            }
            $message = "Failed to get the customer details from recharge";

            $this->helperObj->logMe(
                "Failed to get the customer details from recharge: ".
                " | customerHash -> ".$customerHash.
                " | Subscription -> ".$subscriptionId.
                " | Error -> ".$error,
                $this->logFiles['subscriptionReactivationWithCoupon']['filename']
            );
           
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
           
        }

        $rechargeCustomerHash = trim($customerDetails->customer->hash);

        if ($customerHash == $rechargeCustomerHash) {

            $reactivateResponse = $this->rechargeObj->activateSubscription($subscriptionId);

            if (isset($reactivateResponse->error) || isset($reactivateResponse->errors) || empty($reactivateResponse)) {

                $error = "Failed subscriptions - ".$subscriptionId.' - '.$reactivateResponse->error;
                $message = "Failed to reactivate the subscription";
                $status = "Failed";

                $this->helperObj->logMe(
                    "Failed to reactivate the subscription: ".
                    " | customerHash -> ".$customerHash.
                    " | Subscription -> ".$subscriptionId.
                    " | Error -> ".$error,
                    $this->logFiles['subscriptionReactivationWithCoupon']['filename']
                );
               
                $response = array(
                    "status"       => $status,
                    "message" => $message,
                    "error" => $error
                );
                echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                die();
                
            }
            else{

                /* Iterable event for reactivating the subscription*/
                $subscriptionProperties = '';

                if(isset($subscriptionDetails->subscription) && !empty($subscriptionDetails->subscription)){
                    $subscriptionProperties = $subscriptionDetails->subscription->properties;
                }

                if(isset($customerDetails->customer->email) && !empty($customerDetails->customer->email)){

                    $lineItemDetails = [];
                    if(!empty($subscriptionProperties)){
                        foreach($subscriptionProperties as $item) {
                            // print_r($value->name);die;
                            if (isset($item->name) && strpos($item->name, 'Vitamin') !== false) {
                                $explodeValue = explode('|', $item->value);
                                $lineItem = new \stdClass();

                                if(!empty($explodeValue)){
                                    $lineItem->name = $item->name;
                                    $lineItem->value = $explodeValue[0];
                                    $lineItem->variant_id = $explodeValue[3];
                                    $lineItem->quantity = $explodeValue[2];
                                    $lineItem->sku = $explodeValue[4];
                                    $lineItemDetails[] = $lineItem;
                                }

                            }
                        }
                    }

                    $customerEmail = $customerDetails->customer->email;

                    $eventRequest = array(
                        "events" => array(
                            array("email" => strtolower($customerEmail),
                                "eventName" => "subscriptionReactivated",
                                "dataFields" => array("lineItems" => $lineItemDetails)
                            )
                        )
                    );
                    $eventResponse = $this->iterableObj->createNewIterableEvent(json_encode($eventRequest));

                    if(isset($eventResponse->successCount) || $eventResponse->successCount > 0) {
                        $this->helperObj->logMe(
                            "Subscription reactivation event - Successfully created for Email (".
                            strtolower($customerEmail).")".
                            " | customerHash -> ".$customerHash.
                            " | Subscription -> ".$subscriptionId,
                            $this->logFiles['subscriptionReactivationWithCoupon']['filename']
                        );
                    } else {
                        $this->helperObj->logMe(
                            "Subscription reactivation event - Failed to create event for Email (".
                            strtolower($customerEmail).")".
                            " | customerHash -> ".$customerHash.
                            " | Subscription -> ".$subscriptionId,
                            $this->logFiles['subscriptionReactivationWithCoupon']['filename']
                        );
                    }
                }
                /* Iterable event for reactivating the subscription*/

                /* apply coupon for next charge */
                if (isset($receivedData->couponCode) && !empty($receivedData->couponCode)) {

                    $subscriptionChargeDetails = $this->rechargeObj->getChargeDetailsBySubscriptionID($subscriptionId);

                    if (!empty($subscriptionChargeDetails->error) || !empty($subscriptionChargeDetails->errors) ||
                        empty($subscriptionChargeDetails)) {

                        $status = "Failed";
                        $error  = "Unknown error";
                        if (!empty($subscriptionChargeDetails->error)) {

                            $error = $subscriptionChargeDetails->error;

                        } else if (!empty($subscriptionChargeDetails->errors)) {

                            $error = "Failed to get the charge details by using subscriptionId: ".$subscriptionId;
                        }

                        $message = "Failed to get the subscription charge details from recharge";
                        $this->helperObj->logMe(
                            "Failed to get the charge details by using subscriptionId:".
                            " | customerHash -> ".$customerHash.
                            " | Subscription -> ".$subscriptionId.
                            " | Error -> ".$error,
                            $this->logFiles['subscriptionReactivationWithCoupon']['filename']
                        );
                        
                        $response = array(
                            "status"       => $status,
                            "message" => $message,
                            "error" => $error
                        );
                        echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                        die();
                        
                    }
                    $subscriptionChargeStatus = null;
                    $subscriptionchargeID = null;
                    foreach ($subscriptionChargeDetails->charges as $chargeValue) {

                        $subscriptionChargeTags = strtolower(trim($chargeValue->tags));
                        if (strpos($subscriptionChargeTags, 'subscription' ) !== false &&
                        strtolower(trim($chargeValue->status)) == 'queued' ) {

                            $subscriptionchargeID = $chargeValue->id;
                            $subscriptionChargeStatus = $chargeValue->status;
                        }
                    }
                    if (empty($subscriptionChargeStatus)) {

                        $message = "The subscription queued charge not found for apply the discount subscriptionID: "
                        .$subscriptionId;
                        $error = "Queued charge not found";
                        $status = "Failed";

                        $this->helperObj->logMe(
                            "Queued charge not found:".
                            " | customerHash -> ".$customerHash.
                            " | Subscription -> ".$subscriptionId,
                            $this->logFiles['subscriptionReactivationWithCoupon']['filename']
                        );
                        
                        $response = array(
                            "status"  => $status,
                            "message" => $message,
                            "error"   => $error
                        );
                        echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                        die();
                        
                    }


                    $discountCode = trim($receivedData->couponCode);
                    $discountCodeDetails = $this->rechargeObj->getDiscountDetailsByDiscountCode($discountCode);
                    if (!empty($discountCodeDetails->error) || !empty($discountCodeDetails->errors) ||
                    empty($discountCodeDetails)) {

                        $status = "Failed";
                        $error  = "Unknown error";
                        if (!empty($discountCodeDetails->error)) {

                            $error = $discountCodeDetails->error;

                        } else if (!empty($discountCodeDetails->errors)) {

                            $error = "Failed to get the discount code details from recharge";
                        }

                        $message = "Failed to get the discount details from recharge by using the provided discountCode: "
                        .$discountCode;
                        $this->helperObj->logMe(
                            "Failed to get the discount details from recharge by using the provided discountCode:".
                            " | customerHash -> ".$customerHash.
                            " | Subscription -> ".$subscriptionId.
                            " | DiscountCode -> ".$discountCode.
                            " Error -> ".$error,
                            $this->logFiles['subscriptionReactivationWithCoupon']['filename']
                        );
                        
                        $response = array(
                            "status"  => $status,
                            "message" => $message,
                            "error"   => $error
                        );
                        echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                        die();
                        
                    }
                    $discountRequest = array(
                        "discount_code" => $discountCode
                    );

                    $discountApplyResponse = $this->rechargeObj->applyDiscountToChargeByDiscountCode($subscriptionchargeID,
                    json_encode($discountRequest));

                    if (!empty($discountApplyResponse->error) || !empty($discountApplyResponse->errors) ||
                    empty($discountApplyResponse)) {

                        $status = "Failed";
                        $error  = "Unknown error";
                        if (!empty($discountApplyResponse->error)) {

                            $error = $discountApplyResponse->error;

                        } else if (!empty($discountApplyResponse->errors)) {

                            $error = "Failed to apply the discount to the charge";
                        }

                        $message = "Failed to apply the discount to the charge chargeID: ".$subscriptionchargeID;
                        $this->helperObj->logMe(
                            "Failed to apply the discount to the charge: ".
                            " | Subscription -> ".$subscriptionId.
                            " | customerHash -> ".$customerHash.
                            " | chargeID: ".$subscriptionchargeID.
                            " | DiscountCode: ".$discountCode.
                            " | Error -> ".$error,
                            $this->logFiles['subscriptionReactivationWithCoupon']['filename']
                        );
                        $response = array(
                            "status"  => $status,
                            "message" => $message,
                            "error"   => $error
                        );
                        echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                        die();
                       
                    }


                    $message = "Successfully activated the subscription and applied the discount";
                    $error = "";
                    $status = "Success";

                    $this->helperObj->logMe(
                        "Successfully created the subscription and applied the discount.".
                        " | Subscription -> ".$subscriptionId.
                        " | customerHash -> ".$customerHash.
                        " | chargeID: ".$subscriptionchargeID.
                        " | DiscountCode: ".$discountCode,
                        $this->logFiles['subscriptionReactivationWithCoupon']['filename']
                    );
                    
                    $response = array(
                        "status"  => $status,
                        "message" => $message,
                        "error"   => $error
                    );
                    echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                    die();
                   
                }

                /* apply coupon for next charge */
                $this->helperObj->logMe(
                    "Subscription reactivated successfully.".
                    " | customerHash -> ".$customerHash.
                    " | Subscription -> ".$subscriptionId,
                    $this->logFiles['subscriptionReactivationWithCoupon']['filename']
                );
                
                $message = "Subscription reactivated successfully";
                $error = "";
                $status = "Success";
                $response = array(
                        "status"  => $status,
                        "message" => $message,
                        "error"   => $error
                );
                echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                die();
                
            }

        } else {
            $message = "Failed to reactivate the subscription";
            $error = "Customer hash tags are not matched";
            $status = "Failed";
            $this->helperObj->logMe(
                "Failed to reactivate the subscription. Customer hash tags are not matched:".
                " | customerHash -> ".$customerHash.
                " | Subscription -> ".$subscriptionId,
                $this->logFiles['subscriptionReactivationWithCoupon']['filename']
            );
            $response = array(
                "status"  => $status,
                "message" => $message,
                "error"   => $error
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
           
        }
    }
    
    
    /**
     * Adjust subscription order date
    */
    public function adjustNextOrderDate() {

        if (empty($_REQUEST['data'])) {

            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }

        $requestData = $_REQUEST['data'];

        $receivedData = json_decode($requestData);

        //die('under development...');
        $subscriptionId = '';
        $customerHash = '';


        if (!isset($receivedData->subscriptionId) || empty($receivedData->subscriptionId)) {

            $message = "Failed to adjust the order date.";
            $error = "Missing required parameter subscription id";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();

        }

        if (!isset($receivedData->customerHash) || empty($receivedData->customerHash)) {

            $message = "Failed to adjust the order date.";
            $error = "Missing required parameter recharge customer hash";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();

        }

        $subscriptionId = trim($receivedData->subscriptionId);
        $customerHash = trim($receivedData->customerHash);

        $subscriptionDetails = $this->rechargeObj->getSubscription($subscriptionId);

        if (!isset($subscriptionDetails) || isset($subscriptionDetails->error)
         || isset($subscriptionDetails->errors) || empty($subscriptionDetails->subscription->customer_id)) {

            $status = "Failed";
            $error  = "Unknown error";
            if (isset($subscriptionDetails->error)) {

                $error = $subscriptionDetails->error;

            } else if (isset($subscriptionDetails->errors)) {

                $error = $subscriptionDetails->errors;
            }
            $message = "Failed to adjust the order date. Invalid subscription ID";

            $this->helperObj->logMe(
                "Failed to adjust the order date: Invalid subscription ID: ".
                " | customerHash -> ".$customerHash.
                " | Subscription -> ".$subscriptionId.
                " | Error -> ".$error,
                $this->logFiles['adjustNextChargeDate']['filename']
            );

            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();

        }

        $customerDetails = $this->rechargeObj->getRechargeCustomerDetailsByID($subscriptionDetails->subscription->customer_id);

        if (!isset($customerDetails) || isset($customerDetails->error)
         || isset($customerDetails->errors) || empty($customerDetails->customer->hash)) {

            $status = "Failed";
            $error  = "Unknown error";
            if (isset($customerDetails->error)) {

                $error = $customerDetails->error;

            } else if (isset($customerDetails->errors)) {

                $error = $customerDetails->errors;
            }
            $message = "Failed to get the customer details from recharge";

            $this->helperObj->logMe(
                "Failed to get the customer details from recharge: ".
                " | customerHash -> ".$customerHash.
                " | Subscription -> ".$subscriptionId.
                " | Error -> ".$error,
                $this->logFiles['adjustNextChargeDate']['filename']
            );

            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();

        }

        $rechargeCustomerHash = trim($customerDetails->customer->hash);

        if ($customerHash == $rechargeCustomerHash) {

            /*Change next charge date by subscription ID */
            if (empty($subscriptionDetails->subscription->next_charge_scheduled_at)) {

                $message = "The next charge date is not found in the subscription - ".$subscriptionId;
                $error = "Next charge date not found";
                $status = "Failed";
                $response = array(
                    "status"       => $status,
                    "message" => $message,
                    "error" => $error
                );
                $this->helperObj->logMe(
                    "Next charge date not found: ".
                    " | customerHash -> ".$customerHash.
                    " | Subscription -> ".$subscriptionId.
                    " | Error -> ".$error,
                    $this->logFiles['adjustNextChargeDate']['filename']
                );
                echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                die();
            }
            
            $adjustDays = 1;
            $subscriptionCurrentChargeDate = $subscriptionDetails->subscription->next_charge_scheduled_at;
            $subscriptionCurrentChargeDate = date('Y-m-d', strtotime($subscriptionCurrentChargeDate));
            $adjustDays = (int)$adjustDays;
            
            $currentDate = date('Y-m-d');
            if($adjustDays > 0){
                $newChargeDate = date('Y-m-d',strtotime('+'.$adjustDays.' days',strtotime($currentDate)));
                //$newChargeDate = date('Y-m-d',strtotime('+'.$adjustDays.' days',strtotime($subscriptionCurrentChargeDate)));
                // $newChargeDate = date('Y-m-d',strtotime('-1 days',strtotime($subscriptionCurrentChargeDate)));
            }
            $payload = array(
                "date" => $newChargeDate
            );
            
            // $payload = array(
            //     "date" => date('Y-m-d')
            // );
            
            $subscriptionUpdateResponse = $this->rechargeObj->updateSubscriptionChargeDate($subscriptionId, json_encode($payload));

            if (isset($subscriptionUpdateResponse->error) ||
            isset($subscriptionUpdateResponse->errors) || empty($subscriptionUpdateResponse)) {

                $error = "Failed subscription - ".$subscriptionId.' - '.$subscriptionUpdateResponse->error;
                $message = "Failed to change the next charge order date";
                $status = "Failed";

                $this->helperObj->logMe(
                    "Failed to change the next charge order date: ".
                    " | customerHash -> ".$customerHash.
                    " | Subscription -> ".$subscriptionId.
                    " | Error -> ".$error,
                    $this->logFiles['adjustNextChargeDate']['filename']
                );

                $response = array(
                    "status"       => $status,
                    "message" => $message,
                    "error" => $error
                );
                echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                die();
            }
            /*Change next charge date by subscription ID */
            else{

                /* apply coupon for next charge */
                if (isset($receivedData->couponCode) && !empty($receivedData->couponCode)) {

                    $subscriptionChargeDetails = $this->rechargeObj->getChargeDetailsBySubscriptionID($subscriptionId);

                    if (!empty($subscriptionChargeDetails->error) || !empty($subscriptionChargeDetails->errors) ||
                        empty($subscriptionChargeDetails)) {

                        $status = "Failed";
                        $error  = "Unknown error";
                        if (!empty($subscriptionChargeDetails->error)) {

                            $error = $subscriptionChargeDetails->error;

                        } else if (!empty($subscriptionChargeDetails->errors)) {

                            $error = "Failed to get the charge details by using subscriptionId: ".$subscriptionId;
                        }

                        $message = "Failed to get the subscription charge details from recharge";
                        $this->helperObj->logMe(
                            "Failed to get the charge details by using subscriptionId:".
                            " | customerHash -> ".$customerHash.
                            " | Subscription -> ".$subscriptionId.
                            " | Error -> ".$error,
                            $this->logFiles['adjustNextChargeDate']['filename']
                        );

                        $response = array(
                            "status"       => $status,
                            "message" => $message,
                            "error" => $error
                        );
                        echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                        die();
                        // $this->sendResponse("Failed", $message, $error);
                    }
                    $subscriptionChargeStatus = null;
                    $subscriptionchargeID = null;
                    foreach ($subscriptionChargeDetails->charges as $chargeValue) {

                        $subscriptionChargeTags = strtolower(trim($chargeValue->tags));
                        if (strpos($subscriptionChargeTags, 'subscription' ) !== false &&
                        strtolower(trim($chargeValue->status)) == 'queued' ) {

                            $subscriptionchargeID = $chargeValue->id;
                            $subscriptionChargeStatus = $chargeValue->status;
                        }
                    }
                    if (empty($subscriptionChargeStatus)) {

                        $message = "The upcoming charge not found for apply the discount subscriptionID: "
                        .$subscriptionId;
                        $error = "Queued charge not found";
                        $status = "Failed";

                        $this->helperObj->logMe(
                            "Queued charge not found:".
                            " | customerHash -> ".$customerHash.
                            " | Subscription -> ".$subscriptionId,
                            $this->logFiles['adjustNextChargeDate']['filename']
                        );

                        $response = array(
                            "status"  => $status,
                            "message" => $message,
                            "error"   => $error
                        );
                        echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                        die();

                    }


                    $discountCode = trim($receivedData->couponCode);
                    $discountCodeDetails = $this->rechargeObj->getDiscountDetailsByDiscountCode($discountCode);
                    if (!empty($discountCodeDetails->error) || !empty($discountCodeDetails->errors) ||
                    empty($discountCodeDetails)) {

                        $status = "Failed";
                        $error  = "Unknown error";
                        if (!empty($discountCodeDetails->error)) {

                            $error = $discountCodeDetails->error;

                        } else if (!empty($discountCodeDetails->errors)) {

                            $error = "Failed to get the discount code details from recharge";
                        }

                        $message = "Failed to get the discount details from recharge by using the provided discountCode: "
                        .$discountCode;
                        $this->helperObj->logMe(
                            "Failed to get the discount details from recharge by using the provided discountCode:".
                            " | customerHash -> ".$customerHash.
                            " | Subscription -> ".$subscriptionId.
                            " | DiscountCode -> ".$discountCode.
                            " Error -> ".$error,
                            $this->logFiles['adjustNextChargeDate']['filename']
                        );

                        $response = array(
                            "status"  => $status,
                            "message" => $message,
                            "error"   => $error
                        );
                        echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                        die();

                    }
                    $discountRequest = array(
                        "discount_code" => $discountCode
                    );

                    $discountApplyResponse = $this->rechargeObj->applyDiscountToChargeByDiscountCode($subscriptionchargeID,
                    json_encode($discountRequest));

                    if (!empty($discountApplyResponse->error) || !empty($discountApplyResponse->errors) ||
                    empty($discountApplyResponse)) {

                        $status = "Failed";
                        $error  = "Unknown error";
                        if (!empty($discountApplyResponse->error)) {

                            $error = $discountApplyResponse->error;

                        } else if (!empty($discountApplyResponse->errors)) {

                            $error = "Failed to apply the discount to the charge";
                        }

                        $message = "Failed to apply the discount to the charge chargeID: ".$subscriptionchargeID;
                        $this->helperObj->logMe(
                            "Failed to apply the discount to the charge: ".
                            " | Subscription -> ".$subscriptionId.
                            " | customerHash -> ".$customerHash.
                            " | chargeID: ".$subscriptionchargeID.
                            " | DiscountCode: ".$discountCode.
                            " | Error -> ".$error,
                            $this->logFiles['adjustNextChargeDate']['filename']
                        );
                        $response = array(
                            "status"  => $status,
                            "message" => $message,
                            "error"   => $error
                        );
                        echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                        die();
                        // $this->sendResponse($status, $message, $error);
                    }


                    $message = "Successfully changed the charge date of subscription and applied the discount.";
                    $error = "";
                    $status = "Success";

                    $this->helperObj->logMe(
                        "Successfully changes the charge date of subscription and applied the discount.".
                        " | Subscription -> ".$subscriptionId.
                        " | customerHash -> ".$customerHash.
                        " | chargeID: ".$subscriptionchargeID.
                        " | DiscountCode: ".$discountCode,
                        $this->logFiles['adjustNextChargeDate']['filename']
                    );

                    $response = array(
                        "status"  => $status,
                        "message" => $message,
                        "error"   => $error
                    );
                    echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                    die();

                }

                /* apply coupon for next charge */
                $this->helperObj->logMe(
                    "Subscription date changed successfully.".
                    " | customerHash -> ".$customerHash.
                    " | Subscription -> ".$subscriptionId,
                    $this->logFiles['subscriptionReactivationWithCoupon']['filename']
                );

                $message = "Subscription date changed successfully.";
                $error = "";
                $status = "Success";
                $response = array(
                        "status"  => $status,
                        "message" => $message,
                        "error"   => $error
                );
                echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
                die();
            }

        } else {
            $message = "Failed to change the order date.";
            $error = "Customer hash tags are not matched";
            $status = "Failed";
            $this->helperObj->logMe(
                "Failed to change the order date of the subscription. Customer hash tags are not matched:".
                " | customerHash -> ".$customerHash.
                " | Subscription -> ".$subscriptionId,
                $this->logFiles['subscriptionReactivationWithCoupon']['filename']
            );
            $response = array(
                "status"  => $status,
                "message" => $message,
                "error"   => $error
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }
    }
    
}
