<?php

namespace App\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use App\Model\RechargeApi;
use App\Model\IterableApi;
use App\Helper\Helper;

class IterableSubscriptionChargeUpcomingEvent implements ShouldQueue
{
    use InteractsWithQueue, Queueable;

    private $helperObj;
    private $iterableObj;
    private $rechargeObj;
    private $webhookResponse;
    private $logFile;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($webhookResponse, $logPath)
    {
        $this->webhookResponse = $webhookResponse;
        $this->logFile = $logPath;
        $this->iterableObj = new IterableApi();
        $this->rechargeObj = new RechargeApi();
        $this->helperObj = new Helper();
    }

    public function handle()
    {
        $nextChargeDate = '';
        $subscriptionLineItemsArray = [];
        $subscriptionLineItems = '';
        if(!empty($this->webhookResponse['charge']['line_items'])) {

            $nextChargeDate = !empty($this->webhookResponse['charge']['scheduled_at']) ?
            $this->webhookResponse['charge']['scheduled_at'] : '';

            foreach ($this->webhookResponse['charge']['line_items'] as $chargeValue) {

                if(!empty($chargeValue['properties'])) {

                    foreach ($chargeValue['properties'] as $property) {

                        $propertyName = strtolower(trim($property['name']));

                        if (strpos($propertyName, 'vitamin') !== false) {

                            $propertyValue = trim($property['value']);

                            if(strpos($propertyValue, '|') !== false) {

                                $explodePropertyValue = explode('|', $propertyValue);

                                if(count($explodePropertyValue) == 5) {

                                    $subscriptionLineItemsArray[] = trim($explodePropertyValue[0]);

                                }
                            }
                        }
                    }
                }
            }
        }
        
        
        $firstName = '';
        if(isset($this->webhookResponse['charge']['first_name']) && !empty($this->webhookResponse['charge']['first_name'])){{
            $firstName = $this->webhookResponse['charge']['first_name'];
        }
        
        $discountCode = '';
        if(isset($this->webhookResponse['charge']['discount_codes']) && !empty($this->webhookResponse['charge']['discount_codes'])){
            
            $discount = $this->webhookResponse['charge']['discount_codes'][0];
            $discountCode = $discount['code'];
            
        }
        
        $totalPrice = '';
        if(isset($this->webhookResponse['charge']['total_price']) && !empty($this->webhookResponse['charge']['total_price'])){{
            $totalPrice = $this->webhookResponse['charge']['total_price'];
        }
        
        

        if (!isset($subscriptionLineItemsArray) || empty($subscriptionLineItemsArray) ||
        empty($this->webhookResponse['charge']['email'])) {

            if(empty($this->webhookResponse['charge']['email'])) {

                $error = "Upcoming Charge iterable event - The email address is not valid or the field is empty";
            } else {

                $error = "Upcoming Charge iterable event - The charge line items are not valid or the line
                items are empty(Email - ".$this->webhookResponse['charge']['email'].")";
            }
            //$status = "failed";
            $this->helperObj->logMe($error, $this->logFile);
            die();
        }

        if (!empty($nextChargeDate)) {

            $nextChargeDate = strtotime($nextChargeDate);
            $nextChargeDate = date("Y-m-d H:i:s", $nextChargeDate);
        }
        $subscriptionLineItems = implode(' | ', $subscriptionLineItemsArray);

        $customerID = $this->webhookResponse['charge']['customer_id'];
        $customerSubscriptionDetails = $this->rechargeObj->getAllActiveSubscriptionsByCustomerID($customerID);

        $contactPropertyNextChargeDate = $nextChargeDate;
        $contactPropertySubscriptionItems = $subscriptionLineItems;

        if (!isset($customerSubscriptionDetails->subscriptions)) {
            if (isset($customerSubscriptionDetails->error) || isset($customerSubscriptionDetails->errors)) {
                if (isset($customerSubscriptionDetails->error)) {
                    $error = json_encode($customerSubscriptionDetails->error);
                } else if (isset($customerSubscriptionDetails->errors)) {
                    $error = json_encode($customerSubscriptionDetails->errors);
                }
                $this->helperObj->logMe("Failed to fetch subscription details from recharge, Error is: " . $error, $this->logFile);
            } else {
                $this->helperObj->logMe("Failed to fetch subscription details from recharge, Error details not available", $this->logFile);
            }
        } else {
            if (count($customerSubscriptionDetails->subscriptions) > 1) {
                $contactPropertyNextChargeDate = NULL;
                $contactPropertySubscriptionItems = "Multiple Subscriptions";
            }
        }

        $eventRequest = array(
            "events" => array(
            array("email" => strtolower($this->webhookResponse['charge']['email']),
            "eventName" => "Upcoming Charge",
            "dataFields" => array(
                "nextChargeDate" => $nextChargeDate,
                "subscriptionItems" => $subscriptionLineItems,
                "totalPrice" => $totalPrice,
                "firstName" => $firstName,
                "couponCode" => $discountCode
                )
              )
            )
        );

        $userIterableUpdateRequest = array(
            "email" => strtolower($this->webhookResponse['charge']['email']),
            "dataFields" => array(
                "nextChargeDate" => $contactPropertyNextChargeDate,
                "subscriptionItems" => $contactPropertySubscriptionItems
            )
        );

        $userIterableUpdateResponse = $this->iterableObj->updateUserDataFields(json_encode($userIterableUpdateRequest));

        if(strtolower(trim($userIterableUpdateResponse->code)) == 'success') {

            $this->helperObj->logMe("Charge/Upcoming user contact update - Successfully updated the charge nextChargeDate and
            subscriptionItems in Iterable user contact details(Email - ".strtolower($this->webhookResponse['charge']['email']).")", $this->logFile);
        } else {
            $this->helperObj->logMe("Charge/Upcoming user contact update - Failed to update the charge nextChargeDate and
            subscriptionItems in Iterable user contact details (Email  - ".
            strtolower($this->webhookResponse['charge']['email']).")", $this->logFile);
        }

        $eventResponse = $this->iterableObj->createNewIterableEvent(json_encode($eventRequest));

        if(isset($eventResponse->successCount) || $eventResponse->successCount > 0) {

            $this->helperObj->logMe("Upcoming Charge iterable event - Successfully created event for Email (".
            strtolower($this->webhookResponse['charge']['email']).")", $this->logFile);
        } else {
            $this->helperObj->logMe("Upcoming Charge iterable event - Failed to create event for Email (".
            strtolower($this->webhookResponse['charge']['email']).")", $this->logFile);
        }
    }
}
